//-----------------------------------------------------------------------------

// Variables used by client scripts & code.  The ones marked with (c)
// are accessed from code.  Variables preceeded by Pref:: are client
// preferences and stored automatically in the ~/client/prefs.cs file
// in between sessions.
//
//    (c) Client::MissionFile             Mission file name
//    ( ) Client::Password                Password for server join

//    (?) Pref::Player::CurrentFOV
//    (?) Pref::Player::DefaultFov
//    ( ) Pref::Input::KeyboardTurnSpeed

//    (c) pref::Master[n]                 List of master servers
//    (c) pref::Net::RegionMask     
//    (c) pref::Client::ServerFavoriteCount
//    (c) pref::Client::ServerFavorite[FavoriteCount]
//    .. Many more prefs... need to finish this off

// Moves, not finished with this either...
//    (c) firstPerson
//    $mv*Action...

//-----------------------------------------------------------------------------
// These are variables used to control the shell scripts and
// can be overriden by mods:
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// loadMaterials - load all materials.cs files
//-----------------------------------------------------------------------------
function loadMaterials()
{
   for( %file = findFirstFile( "*/materials.cs" ); %file !$= ""; %file = findNextFile( "*/materials.cs" ))
   {
      exec( %file );
   }
}

function reloadMaterials()
{
   reloadTextures();
   loadMaterials();
   reInitMaterials();
}


//-----------------------------------------------------------------------------
function initClient()
{
   echo("\n--------- Initializing FPS: Client ---------");

   // Make sure this variable reflects the correct state.
   $Server::Dedicated = false;

   // Game information used to query the master server
   $Client::GameTypeQuery = "Test App";
   $Client::MissionTypeQuery = "Any";

   exec("./ui/customProfiles.cs"); // override the base profiles if necessary
   
   // The common module provides basic client functionality
   //initBaseClient();
   setRandomSeed();
   exec("./actionMap.cs");
   exec("./audio.cs");
   exec("./canvas.cs");
   exec("./cursor.cs");
   exec("./help.cs");
   exec("./materials.cs");
   exec("./message.cs");
   exec("./messageBox.cs");
   //exec("./metrics.cs");
   exec("./missionDownload.cs");
   exec("./recordings.cs");
   exec("./screenshot.cs");
   //exec("./shaders.cs");

   // InitCanvas starts up the graphics system.
   // The canvas needs to be constructed before the gui scripts are
   // run because many of the controls assume the canvas exists at
   // load time.
   initCanvas("Legions Prototype");

   /// Load client-side Audio Profiles/Descriptions
   exec("./scripts/audioProfiles.cs");

   // Load up the Game GUIs
   exec("./ui/defaultGameProfiles.cs");
   exec("./ui/PlayGui.gui");

   // Load up the shell GUIs
   exec("./ui/mainMenuGui.gui");
   exec("./ui/aboutDlg.gui");
   exec("./ui/startMissionGui.gui");
   exec("./ui/joinServerGui.gui");
   exec("./ui/endGameGui.gui");
   exec("./ui/loadingGui.gui");
   exec("./ui/optionsDlg.gui");
   exec("./ui/remapDlg.gui");

   // Client scripts
   exec("./scripts/client.cs");
   exec("./scripts/missionDownload.cs");
   exec("./scripts/serverConnection.cs");
   exec("./scripts/playerList.cs");
   exec("./scripts/loadingGui.cs");
   exec("./scripts/optionsDlg.cs");
   exec("./scripts/messageHud.cs");
   exec("./scripts/playGui.cs");
   exec("./scripts/centerPrint.cs");
   exec("./scripts/game.cs");

   exec("./scripts/clientInventory.cs");

   // material related
   exec("./scripts/glowBuffer.cs");
   //exec("common/client/shaders.cs");
   exec("./scripts/shaders.cs");
   exec("./scripts/commonMaterialData.cs" );
   
   // Default player key bindings
   exec("./scripts/default.bind.cs");
   exec("./config.cs");


   //team selection
   exec("./ui/SelectTeamGui.gui");
   exec("./scripts/SelectTeamGui.cs");

   //player class selection
   exec("./ui/SelectPlayerClassGui.gui");
   exec("./scripts/SelectPlayerClassGui.cs");

   //flag status
   exec("./ui/flagStatusGui.gui");
   exec("./scripts/flagStatusGui.cs");

   //score
   exeC("./ui/scoreGui.gui");
   exeC("./scripts/scoreGui.cs");

   //chat
   exec("./ui/newChatHud.gui");
   exec("./scripts/newChatHud.cs");
   exec("./ui/newMessageHud.gui");
   exec("./scripts/newMessageHud.cs");
   exec("./scripts/whostalking.cs");

   //MOTD
   exec("./ui/MOTDGui.gui");
   exec("./scripts/MOTDGui.cs");

   loadMaterials();
   initRenderInstManager();
   
   // Really shouldn't be starting the networking unless we are
   // going to connect to a remote server, or host a multi-player
   // game.
   setNetPort(0);

   // Copy saved script prefs into C++ code.
   setShadowDetailLevel( $pref::shadows );
   setDefaultFov( $pref::Player::defaultFov );
   setZoomSpeed( $pref::Player::zoomSpeed );

   // Start up the main menu... this is separated out into a 
   // method for easier mod override.
   loadMainMenu();

   //loadDefaultMission();

   // Connect to server if requested.
   if ($JoinGameAddress !$= "") {
      connect($JoinGameAddress, "", $Pref::Player::Name);
   }
}


//-----------------------------------------------------------------------------
function loadDefaultMission()
{
   Canvas.setCursor("DefaultCursor");
   createServer( "SinglePlayer", "base/data/missions/ctf/zoneControl.mis" );

   %conn = new GameConnection(ServerConnection);
   RootGroup.add(ServerConnection);
   %conn.setConnectArgs($pref::Player::Name);
   %conn.setJoinPassword($Client::Password);
   %conn.connectLocal();
}

function startListenServer()
{
   Canvas.setCursor("DefaultCursor");
   createServer( "MultiPlayer", "base/data/missions/ctf/zoneControl.mis" );

   %conn = new GameConnection(ServerConnection);
   RootGroup.add(ServerConnection);
   %conn.setConnectArgs($pref::Player::Name);
   %conn.setJoinPassword($Client::Password);
   %conn.connectLocal();
}

function loadMainMenu()
{
   // Startup the client with the Main menu...
   Canvas.setContent( MainMenuGui );
   Canvas.setCursor("DefaultCursor");
}

function checkVideoInit()
{
   if($GFX::OutdatedDrivers)
   {
      MessageBoxOK("Old Video Drivers",
         "You have old video drivers.  " @
         "TGEA uses many advanced rendering features, and older drivers may not support them correctly. " @
         "YOU SHOULD UPDATE YOUR DRIVERS BEFORE PROCEEDING.  " @
         $GFX::OutdatedDriversLink);
   }
}
