//constants.cs

$pi = 3.141597;


$Game::TeamScoreCap     = 10;   // default score limit (for when a map does not define one)

$Game::SpawnFireDelay   = 200;  // how long from when you respawn until when you can shoot
$Game::MinRespawnDelay  = 1  * 1000; // how long from when you die until you can respawn
$Game::MaxRespawnDelay  = 3 * 1000;
$Game::MaxRespawnVelocity = 2;

$Game::FlagAutoReturnTime = 60; //seconds before a dropped flag is automatically returned
$Game::CoreTime = 30 * 1000; 

$Game::EndGamePause = 10; // Pause while looking over the end game screen (in secs)
$Game::Duration     = 20 * 60; // Game duration in secs, no limit if the duration is set to 0
$Game::EndGameScore = 30; // When a client score reaches this value, the game is ended. (not used in team games)

$Game::TeamSwitchDelay = 3 * 1000;


$MaxMountedImages = 4;
$WeaponImageSlot = 0;
$FlagImageSlot = 2;


function eulerToMatrix( %euler )
{
	%euler = VectorScale(%euler, $pi / 180);				//convert euler rotations to radians

	%matrix = MatrixCreateFromEuler(%euler);				//make a rotation matrix
	%xvec = getWord(%matrix, 3);						//get the parts of the matrix you need
	%yvec = getWord(%matrix, 4);
	%zvec = getWord(%matrix, 5);
	%ang  = getWord(%matrix, 6);						//this is in radians
	%ang = %ang * 180 / $pi;						//convert back to degrees

	%rotationMatrix = %xvec @ " " @ %yvec @ " " @ %zvec @ " " @ %ang;	//put it all together

	return %rotationMatrix;							//send it back
}

function getLine(%phrase, %lineNum)
{
	%offset = 0;
	%lineCount = 0;
	while(%lineCount <= %lineNum)
	{
		%pos = strPos(%phrase, "\n", %offset);

		if(%pos >= 0)
			%len = (%pos - %offset);
		else
			%len = 99999;
		%line = getSubStr(%phrase, %offSet, %len);
		
		if(%lineCount == %lineNum)
			return(%line);
		
		%lineCount++;
		%offset = %pos + 1;

		//didn't find anything
		if(%pos == -1)
			return ("");
	}

	return ("");
}

function getLineCount(%phrase)
{
	%offset = 0;
	%lineCount = 0;
	
	while(%offset >= 0)
	{
		%offset = strPos(%phrase, "\n", %offset + 1);

		%lineCount++;
	}

	return(%lineCount);
}

// --------------------------------------------
// miscellaneous yet handy functions - stolen from t2

function posFromTransform(%transform)
{
   // the first three words of an object's transform are the object's position
   %position = getWord(%transform, 0) @ " " @ getWord(%transform, 1) @ " " @ getWord(%transform, 2);
   return %position;
}

function rotFromTransform(%transform)
{
   // the last four words of an object's transform are the object's rotation
   %rotation = getWord(%transform, 3) @ " " @ getWord(%transform, 4) @ " " @ getWord(%transform, 5) @ " " @ getWord(%transform, 6);
   return %rotation;
}

function posFromRaycast(%transform)
{
   // the 2nd, 3rd, and 4th words returned from a successful raycast call are the position of the point
   %position = getWord(%transform, 1) @ " " @ getWord(%transform, 2) @ " " @ getWord(%transform, 3);
   return %position;
}

function normalFromRaycast(%transform)
{
   // the 5th, 6th and 7th words returned from a successful raycast call are the normal of the surface
   %norm = getWord(%transform, 4) @ " " @ getWord(%transform, 5) @ " " @ getWord(%transform, 6);
   return %norm;
}

//rounds to the nearest .01
function round(%val)
{
	%val *= 100;
	%val = mFloor(%val);
	%val /= 100;

	return %val;
}

//takes a time in seconds and returns a string like xxHours xxMinutes xxSeconds
function getTimeString(%timeS)
{
	if (%timeS >= 3600)
	{
		//need hours
		%hours = mFloor(%timeS / 3600);
		%timeS -= %hours * 3600;

		%minutes = mFloor(%timeS / 60);
		%timeS -= %minutes * 60;

		%seconds = %timeS;
		
		if(%minutes < 10)
			%minutes = "0" @ %minutes;
		if(%seconds < 10)
			%seconds = "0" @ %seconds;

		return(%hours @ ":" @ %minutes @ ":" @ %seconds);
		//return(%hours @ " hours, " @ %minutes @ " minutes, " @ %seconds @ "seconds");
	}
	else if(%timeS >= 60)
	{
		//need minutes
		%minutes = mFloor(%timeS / 60);
		%timeS -= %minutes * 60;

		%seconds = %timeS;

		if(%seconds < 10)
			%seconds = "0" @ %seconds;

		return(%minutes @ ":" @ %seconds);
		//return(%minutes @ " minutes, " @ %seconds @ "seconds");
	}
	else
	{
		%seconds = %timeS;

		if(%seconds < 10)
			%seconds = "0" @ %seconds;

		//just seconds
		return("0:" @ %seconds);
		//return(%timeS @ " seconds");
	}
}

//tells you if this is we're hosting a listen server
function isListenServer()
{
   if(isObject(ServerGroup) && isObject(ServerConnection))
      return true;
   else
      return false;
}
