//core.cs

//core item for capture-the-core style gameplay

datablock ParticleData(CoreTrailParticle)
{
   dragCoeffiecient     = 0.0;
   gravityCoefficient   = -0.2;   // rises slowly
   inheritedVelFactor   = 0.05;

   lifetimeMS           = 700;  // lasts 2 second
   lifetimeVarianceMS   = 150;   // ...more or less

   textureName          = "~/data/textures/effects/particle/bubble01";

   useInvAlpha = false;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0]     = "1.0 0.9 0.0 1.0";
   colors[1]     = "1.0 0.6 0.0 1.0";
   colors[2]     = "0.0 0.4 0.0 0.0";

   sizes[0]      = 0.25;
   sizes[1]      = 0.15;
   sizes[2]      = 0.00;

   times[0]      = 0.0;
   times[1]      = 0.2;
   times[2]      = 1.0;

};

datablock ParticleEmitterData(CoreTrailEmitter)
{
   ejectionPeriodMS = 1;
   periodVarianceMS = 0;

   ejectionVelocity = 0.0;
   velocityVariance = 0.0;

   thetaMin         = 0.0;
   thetaMax         = 0.0;  

   particles = "CoreTrailParticle";
};


datablock GrenadeProjectileData( Core0Projectile : MortarProjectile )
{  
   projectileShapeName  = "~/data/shapes/weapons/disc_shell.dts";
   explosion   = GroundMajorExplosion;
   emitter     = CoreTrailEmitter;
   lightColor = "0 0 0 0";
   armingDelay = 10;
   lifetime    = 50;
   elasticity = 0.01;
};

datablock GrenadeProjectileData( Core1Projectile : MortarProjectile )
{  
   projectileShapeName  = "~/data/shapes/weapons/disc_shell.dts";
   explosion   = GroundMajorExplosion;
   emitter     = CoreTrailEmitter;
   lightColor = "0 0 0 0";
   armingDelay = 100;
   lifetime    = 150;
   elasticity = 0.01;
};
datablock GrenadeProjectileData( Core2Projectile : MortarProjectile )
{  
   projectileShapeName  = "~/data/shapes/weapons/disc_shell.dts";
   explosion   = GroundMajorExplosion;
   emitter     = CoreTrailEmitter;
   lightColor = "0 0 0 0";
   armingDelay = 200;
   lifetime    = 250;
   elasticity = 0.01;
};
datablock GrenadeProjectileData( Core3Projectile : MortarProjectile )
{
   projectileShapeName  = "~/data/shapes/weapons/disc_shell.dts";
   explosion   = GroundMajorExplosion;
   emitter     = CoreTrailEmitter;
   lightColor = "0 0 0 0";
   armingDelay = 300;
   lifetime    = 350;
   elasticity = 0.01;
};
datablock GrenadeProjectileData( Core4Projectile : MortarProjectile )
{
   projectileShapeName  = "~/data/shapes/weapons/disc_shell.dts";
   explosion   = GroundMajorExplosion;
   emitter     = CoreTrailEmitter;
   lightColor = "0 0 0 0";
   armingDelay = 400;
   lifetime    = 450;
   elasticity = 0.01;
};
datablock GrenadeProjectileData( Core5Projectile : MortarProjectile )
{
   projectileShapeName  = "~/data/shapes/weapons/disc_shell.dts";
   explosion   = GroundMajorExplosion;
   emitter     = CoreTrailEmitter;
   lightColor = "0 0 0 0";
   armingDelay = 500;
   lifetime    = 550;
   elasticity = 0.01;
};
datablock GrenadeProjectileData( Core6Projectile : MortarProjectile )
{
   projectileShapeName  = "~/data/shapes/weapons/disc_shell.dts";
   explosion   = GroundMajorExplosion;
   emitter     = CoreTrailEmitter;
   lightColor = "0 0 0 0";
   armingDelay = 600;
   lifetime    = 650;
   elasticity = 0.01;
};


datablock ItemData( BlueCoreItem )
{
    category         = "Game";
    className        = "Core";
    dynamicType      = $TypeMasks::DamagableItemObjectType;

    shapeFile        = "~/data/shapes/items/flag_blue.dts";
    mass             = 50;
    elasticity       = 0.2;
    friction         = 0.6;

    throwForce       = 1000;

    flagImage = BlueCoreImage;
};
datablock ShapeBaseImageData( BlueCoreImage )
{
   className      = CoreImageClass;
   isFlag         = true; //respond to drop flag command?
   shapeFile      = "~/data/shapes/items/flag_blue.dts";
   mountPoint     = 2;
};


datablock ItemData( RedCoreItem : BlueCoreItem )
{
    shapeFile        = "~/data/shapes/items/flag_Red.dts";
    flagImage = RedCoreImage;
};
datablock ShapeBaseImageData( RedCoreImage : BlueCoreImage)
{
   shapeFile      = "~/data/shapes/items/flag_red.dts";
};


function Core::onAdd(%this, %obj)
{
   if(%obj.isStatic())
      %obj.status = "Safe";
   
   %obj.setScale("1 1 1");
   //schedule explosion cycle
   %obj.explodeCycle($Game::CoreTime);
}

function Core::ExplodeCycle(%this, %obj, %time)
{
   if(isEventPending(%obj.explodeEvent))
      cancel(%obj.explodeEvent);
   
   %obj.explodeEvent = %obj.schedule(%time, explode);
}
function Core::Explode(%this, %obj)
{
   %currSize = mFloor(getWord(%obj.getScale(), 0));

   //if someone is holding it, or it's dropped in the field, explode
      //explode based on current size of the core
   //otherwise just give the team a point and increase the size of the core
   if(%obj.isHidden())
   {
      if(isObject(%obj.droppedItem))
      {
         //echo("item");
         %sourceObj = %obj.droppedItem;
         %pos = %obj.droppedItem.getWorldBoxCenter();
      }
      else if(isObject(%obj.carryingPlayer))
      {
         //echo("player");
         %sourceObj = %obj.carryingPlayer;
         %pos = %obj.carryingPlayer.getWorldBoxCenter();
      }
      else
      {
         error("ERROR: Core::Explode() - object \"" @ %obj @ "\" has no droppedItem or carryingPlayer");
      }

      //explode!
      %p = new GrenadeProjectile()
      {
         dataBlock = Core0Projectile;
         initialPosition = %pos;
         initialVelocity = "0 0 0";
         sourceObject = %sourceObj;
         //sourceSlot = 0;
      };
      MissionCleanup.add(%p);

      for(%i = 0; %i < %currSize; %i++)
      {
         %x = getRandom() - 0.5;
         %y = getRandom() - 0.5;
         %z = (getRandom() * 0.5) - 0.25;
         %vec = %x SPC %y SPC %z;
         %vec = vectorNormalize(%vec);
         %mag = 10 * %currSize;
         if(%mag < 40)
            %mag = 40;
         %vec = vectorScale(%vec, %mag);

         %data = "core" @ (getRandom(5) + 1) @ "Projectile";

         %p = new GrenadeProjectile()
         {
            dataBlock = %data;
            initialPosition = %pos;
            initialVelocity = %vec;
            sourceObject = %sourceObj;
            //sourceSlot = 0;
         };
         MissionCleanup.add(%p);
      }
   }
   else
   {
      //exploded on stand
      TeamManager_SO.incScore(%obj.team, 1);  
      %currSize++;
   }
   
   %obj.setScale(%currSize SPC %currSize SPC %currSize);
   %obj.reset();

   //schedule next explode event
   %obj.ExplodeCycle($Game::CoreTime);
   %obj.transmitStatus();
}

function Core::GamePreProcess(%this, %obj)
{
   TeamManager_SO.addFlag(%obj);
}

function Core::OnPickup(%this,%obj,%player,%amount)
{
   if(%obj.ismounted())
      return;

   //echo("trying to pick up Core");
   //echo("Core = ", %obj);
   //echo("player = ", %player);

   //if we touch our own flag and it's not static, the the flag is returned (no matter if we're carrying a flag or not)
   if(!%obj.isStatic())
   {
      if(%obj.team == %player.team)
      {
         echo(getTaggedString(%player.client.name) @ " RETURNED " @ %obj.spawn.getName());

         %obj.reset();
         return;
      }
   }
   
   //are we carrying a flag?
   %image = %player.getMountedImage($FlagImageSlot);
   if(isObject(%image))
      if(%image.isFlag == true)
         return;

   //cant pickup our own flag
   if(%player.team == %obj.team)
      return;

   //pickup flag
   %player.mountImage(%this.flagImage,$FlagImageSlot);
   if(%obj.isStatic())
   {  
      %player.client.incStat("Flag_StandGrabs");
      //echo(getTaggedString(%player.client.name) @ " STAND GRABBED " @ %player.flagspawn.getName());

      //picking up flag at base
      %player.flagTeam = %obj.team;
      %player.flagSpawn = %obj;
      %player.flagDB = %this;
      
      %player.flagSpawn.setHidden(1);   
      %player.flagSpawn.carryingPlayer = %player;
      %player.flagSpawn.status = "Taken";
      %player.flagSpawn.transmitStatus();
   }
   else
   {
      //don't count when it was dropped with ctrl F
      if(!%obj.wasPassed)
         %player.client.incStat("Flag_FieldGrabs");
      //echo(getTaggedString(%player.client.name) @ " FIELD GRABBED " @ %player.flagspawn.getName());

      //picking up flag that has been dropped
      %player.flagTeam = %obj.team;
      %player.flagSpawn = %obj.spawn;
      %player.flagDB = %this;

      %obj.delete();
      
      %player.flagSpawn.carryingPlayer = %player;
      %player.flagSpawn.status = "Taken";
      %player.flagSpawn.transmitStatus();
   }
   
   switch(%player.client.team)
   {
      case 0:
         %data = Red_GimpMaleArmor;
      case 1:
         %data = Blue_GimpMaleArmor;
   }
   %player.oldDataBlock = %player.getDataBlock();
   echo("recording old datablock: ", %player.oldDataBlock.getName());
   %player.setDataBlock(%data);
}


function Core::TransmitStatus(%this, %obj, %client)
{
   if(!%obj.isStatic())
   {
      error("ERROR: Core::TransmitStatus() - attempting to send status of non-static object \"" @ %obj @ "\"");
      return;
   }  
   
   %timeLeft = getEventTimeLeft(%obj.explodeEvent);

   switch$(%obj.status)
   {
      case "Safe":
         %status = "Safe" TAB "COUNTDOWN" TAB %timeLeft;
      case "Taken":
         %status = "Taken" TAB "COUNTDOWN" TAB %timeLeft;
      case "Dropped":
         %status = "Dropped" TAB "COUNTDOWN" TAB %timeLeft;
      default:
         %status = "Unknown";
   }

   if(%client $= "")
      messageAll('FlagStatus', '', %obj, %status);
   else
      messageClient(%client, 'FlagStatus', '', %obj, %status);
}

function Core::Reset(%data, %obj)
{
   //echo("FlagItem resetting ", %data, " ", %obj);
   if(%obj.isStatic())
   {
      %spawnItem = %obj;
      if(%obj.isHidden() == true)
      {
         if(isObject(%obj.droppedItem))         //is it dropped?
            %obj.droppedItem.delete();
         else if(isObject(%obj.carryingPlayer)) //is there a player carrying it?
            %obj.carryingPlayer.unMountImage($FlagImageSlot);
         else
            error("ERROR: Core::Reset() - no dropped item or carrying player for " @ %obj.getName() );
      }
   }
   else
   {
      %spawnItem = %obj.spawn;  
      %obj.delete();      
   }

   if(isObject(%spawnItem))
   {
      %spawnItem.status = "Safe";
      %spawnItem.transmitStatus();
      %spawnItem.setHidden(0);
   }
   else
   {
      error("ERROR: Core::Reset() - no spawn object for flag " @ %obj);
   }
}  


function CoreImageClass::onMount(%data)
{
   //echo("FlagImage onmount");
}

function CoreImageClass::OnPlayerDeath(%data, %player, %slot)
{
   //echo("flag holder died ", %player);
   %player.unMountImage(%slot);

   %flag = CoreImageClass::DropFlag(%data,%player);  
   %flag.setCollisionTimeout(%player);
}

function CoreImageClass::DropFlag(%data, %player, %slot)
{
   %droppedCore = FlagImageClass::DropFlag(%data, %player, %slot);
   %droppedCore.setScale(%droppedCore.spawn.getScale());

   echo("returning player to datablock: ", %player.oldDataBlock.getName());
   echo("  player = ", %player);
   %player.setDataBlock(%player.oldDataBlock);

   return %droppedCore;
}



function CoreImageClass::OnPlayerRemove(%data, %player, %slot)
{
   CoreImageClass::DropFlag(%data,%player, %slot);  
}




datablock PlayerData(Blue_GimpMaleArmor : Blue_LightMaleArmor)
{
   jetForce = 0;
   jetEnergyDrain = 0;
}; 
datablock PlayerData(Red_GimpMaleArmor : Red_LightMaleArmor)
{
   jetForce = 0;
   jetEnergyDrain = 0;
};