//flag.cs

//a thing to be captured...

datablock ItemData( BlueFlagItem )
{
    category         = "Game";
    className        = "Flag";
    dynamicType      = $TypeMasks::DamagableItemObjectType;

    shapeFile        = "~/data/shapes/items/flag_blue.dts";
    mass             = 50;
    elasticity       = 0.2;
    friction         = 0.6;

    throwForce       = 1000;

    flagImage = BlueFlagImage;
};
datablock ShapeBaseImageData( BlueFlagImage )
{
   className      = FlagImageClass;
   isFlag         = true; //respond to drop flag command?
   shapeFile      = "~/data/shapes/items/flag_blue.dts";
   mountPoint     = 2;
};


datablock ItemData( RedFlagItem : BlueFlagItem)
{
    shapeFile        = "~/data/shapes/items/flag_Red.dts";
    flagImage = RedFlagImage;
};
datablock ShapeBaseImageData( RedFlagImage : BlueFlagImage )
{
   shapeFile      = "~/data/shapes/items/flag_red.dts";
};


function Flag::onAdd(%this, %obj)
{
   if(%obj.isStatic())
      %obj.status = "Safe";
}

function Flag::GamePreProcess(%this, %obj)
{
   TeamManager_SO.addFlag(%obj);
}

function Flag::OnPickup(%this,%obj,%player,%amount)
{
   if(%obj.ismounted())
      return;

   //echo("trying to pick up flag");
   //echo("flag = ", %obj);
   //echo("player = ", %player);

   //if we touch our own flag and it's not static, the the flag is returned (no matter if we're carrying a flag or not)
   if(!%obj.isStatic())
   {
      if(%obj.team == %player.team)
      {
         echo(getTaggedString(%player.client.name) @ " RETURNED " @ %obj.spawn.getName());

         %obj.reset();
         return;
      }
   }
   
   //are we carrying a flag?
   %haveFlag = false;
   for(%i = 0; %i < $MaxMountedImages; %i++)
   {
      %image = %player.getMountedImage(%i);
      if(!isObject(%image))
         continue;
      
      if(%image.isFlag != true)
         continue;

      if(%image.className $= "FlagImageClass")
      {
         %haveFlag = true;
         %flagSlot = %i;
         break;
      }
      else
      {
         return; //we're carrying some non-ctf game object, bail
      }
   }

   if(%haveFlag)
   {
      //if we're touching our own flag, then we just capped
      if(%player.team != %obj.team)
         return;
      
      //captured
      //echo("CAPTURED!");
      %name = %player.client.getColoredName();
      echo(%name @ " \c2CAPTURED " @ %player.flagspawn.getName());
      messageAll('', %name @ " \c2CAPTURED " @ %player.flagspawn.getName());

      %player.client.incStat("Flag_Captures");
      %player.unMountImagE(%flagSlot);
      %player.flagSpawn.setHidden(0);

      %player.flagSpawn.status = "Safe";
      %player.flagSpawn.transmitStatus();
      //messageAll('FlagStatus', '', %player.flagSpawn, "Safe");
   
      if(%player.flagSpawn.pointValue <= 0)
         %player.flagSpawn.pointValue = 1;

      TeamManager_SO.IncScore(%player.team, %player.flagSpawn.pointValue);
   }
   else
   {
      //cant pickup our own flag
      if(%player.team == %obj.team)
         return;

      //pickup flag
      %player.mountImage(%this.flagImage,$FlagImageSlot);
      if(%obj.isStatic())
      {
         %player.client.incStat("Flag_StandGrabs");
         //echo(getTaggedString(%player.client.name) @ " STAND GRABBED " @ %player.flagspawn.getName());

         //picking up flag at base
         %player.flagTeam = %obj.team;
         %player.flagSpawn = %obj;
         %player.flagDB = %this;
         
         %player.flagSpawn.setHidden(1);  
         %player.flagSpawn.carryingPlayer = %player;
         %player.flagSpawn.status = "Taken";
         %player.flagSpawn.transmitStatus();
      }
      else
      {
         //don't count when it was dropped with ctrl F
         if(!%obj.wasPassed)
            %player.client.incStat("Flag_FieldGrabs");
         //echo(getTaggedString(%player.client.name) @ " FIELD GRABBED " @ %player.flagspawn.getName());

         //picking up flag that has been dropped
         %player.flagTeam = %obj.team;
         %player.flagSpawn = %obj.spawn;
         %player.flagDB = %this;

         %obj.delete();

         %player.flagSpawn.carryingPlayer = %player;
         %player.flagSpawn.status = "Taken";
         %player.flagSpawn.transmitStatus();
      }
   }
  
}


function Flag::TransmitStatus(%this, %obj, %client)
{
   if(!%obj.isStatic())
   {
      error("ERROR: Flag::TransmitStatus() - attempting to send status of non-static object \"" @ %obj @ "\"");
      return;
   }  
   
   switch$(%obj.status)
   {
      case "Safe":
         %status = "Safe";
      case "Taken":
         %status = "Taken";
      case "Dropped":
         %timeLeft = getEventTimeLeft(%obj.resetEvent);
         %status = "Dropped" TAB "COUNTDOWN" TAB %timeLeft;
      default:
         %status = "Unknown";
   }

   if(%client $= "")
      messageAll('FlagStatus', '', %obj, %status);
   else
      messageClient(%client, 'FlagStatus', '', %obj, %status);
}

function Flag::Reset(%data, %obj)
{
   //echo("FlagItem resetting ", %data, " ", %obj);
   if(isObject(%obj.spawn))
   {
      %obj.spawn.status = "Safe";
      %obj.spawn.transmitStatus();
      //messageAll('FlagStatus', '', %obj.spawn, "Safe");
      %obj.spawn.setHidden(0);
      //%obj.schedule(10, delete);      
      %obj.delete();
   }
   else
   {
      error("ERROR: FlagItem::Reset() - no spawn object for flag " @ %obj);
   }
}  


function FlagImageClass::onMount(%data)
{
   //echo("FlagImage onmount");
}

function FlagImageClass::OnPlayerDeath(%data, %player, %slot)
{
   //echo("flag holder died ", %player);
   %player.unMountImage(%slot);

   %flag = FlagImageClass::DropFlag(%data,%player);  
   %flag.setCollisionTimeout(%player);
}

function FlagImageClass::DropFlag(%data, %player)
{
   %flag = new Item()
   {
      dataBlock = %player.flagDB;
   };
   MissionCleanup.add(%flag);
   %flag.team = %player.flagTeam;
   %flag.spawn = %player.flagSpawn;
   %flag.setTransform(%player.getWorldBoxCenter());

   //update the static item's info
   %flag.spawn.droppedItem = %flag; 
   %flag.spawn.carryingPlayer = "";

   if(%player.getDamagePercent() < 1.0)
   {
      %flag.wasPassed = true;
      //echo(getTaggedString(%player.client.name) @ " PASSED " @ %player.flagspawn.getName());
   }
   else
   {
      %player.client.incStat("Flag_Drops");
      //echo(getTaggedString(%player.client.name) @ " DROPPED " @ %player.flagspawn.getName());
   }
   
   %vector = %player.getEyeVector();
   %vector = vectorAdd(%vector, "0 0 0.5");
   %vector = vectorNormalize(%vector);
   %vector = vectorScale(%vector, 15);
   %vector = vectorAdd(%vector, %player.getVelocity());
   %flag.setVelocity(%vector);
   
   %flag.spawn.resetEvent = %flag.schedule($Game::FlagAutoReturnTime * 1000, reset);
   %flag.spawn.status = "Dropped";
   %flag.spawn.transmitStatus();
   //messageAll('FlagStatus', '', %flag.spawn, "Dropped" TAB "COUNTDOWN" TAB $Game::FlagAutoReturnTime);

   return %flag;
}


function FlagImageClass::OnPlayerRemove(%data, %player, %slot)
{
   FlagImageClass::DropFlag(%data,%player);  
}



//drop the flag
function serverCmdDropFlag(%client)
{
   %player = %client.player;
   if(!isObject(%player))
      return;

   for(%i = 0; %i < $MaxMountedImages; %i++)
   {
      %image = %player.getMountedImage(%i);
      if(!isObject(%image))
         continue;
      
      if(%image.isFlag != true)
         continue;

      %image.onPlayerDeath(%player, %i);
      return;
   }
}