//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//  Functions that implement game-play
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function onServerCreated()
{
   // Server::GameType is sent to the master server.
   // This variable should uniquely identify your game and/or mod.
   $Server::GameType = "Test App";

   // Server::MissionType sent to the master server.  Clients can
   // filter servers based on mission type.
   $Server::MissionType = "Deathmatch";

   // GameStartTime is the sim time the game started. Used to calculated
   // game elapsed time.
   $Game::StartTime = 0;

   exec("./constants.cs");

   // Load up all datablocks, objects etc.  This function is called when
   // a server is constructed.
   exec("./audioProfiles.cs");
   exec("./camera.cs");
   exec("./markers.cs"); 
   exec("./triggers.cs"); 
   exec("./shapeBase.cs");
   exec("./staticShape.cs");
   exec("./radiusDamage.cs");
   
   exec("~/server/lightingSystem.cs");

   exec("./serverCmd.cs");
   exec("./consoleCmd.cs");

   //Temp Test player Scipts Below - Tim A
   exec("./player.cs");

   exec("./classNameHacks.cs");
      
   exec("./item.cs");
   exec("./ammo.cs");
   exec("./inventory.cs");
   exec("./projectile.cs");
   exec("./weapon.cs");
   exec("./team.cs");
   exec("./playerClass.cs");
   
   //----------------------------------------
   // Special Effects
   //----------------------------------------
   exec("./effects/bulletImpact_dirt.cs");
   exec("./effects/bulletImpact_rock.cs");
   exec("./effects/bulletImpact_metal.cs");
   exec("./effects/bulletImpact_water.cs");
   exec("./effects/bulletImpact_building.cs");
   exec("./effects/debris_target.cs");
   exec("./effects/explosion_targetMinor.cs");
   exec("./effects/explosion_targetMajor.cs");
   exec("./effects/explosion_groundMinor.cs");
   exec("./effects/explosion_groundMajor.cs");
   exec("./effects/explosion_waterMinor.cs");
   exec("./effects/explosion_waterMajor.cs");
   exec("./effects/explosion_genericSmall.cs");
   exec("./effects/explosion_genericMedium.cs");
   exec("./effects/explosion_genericLarge.cs");
   exec("./effects/explosion_flak.cs");
   exec("./effects/explosion_frag.cs");
   exec("./effects/explosion_molotov.cs");
   //----------------------------------------

   exec("./effects/explosion_genericRedLarge.cs");

   exeC("./lights.cs");

   exec("./weapons/discLauncher.cs");
   exec("./weapons/mortar.cs");
   exec("./weapons/laserExplosionA.cs");
   exec("./weapons/laserExplosionB.cs");
   exec("./weapons/laserExplosionC.cs");
   exec("./weapons/laserRifle.cs");
   exec("./weapons/chaingun.cs");
   exec("./weapons/grenadeLauncher.cs");
   exec("./weapons/shotgun.cs");

   exec("./weapons/turretGun.cs");
   exec("./weapons/healthStationGun.cs");

   exec("./weapons/grenade.cs");
   exec("./weapons/handGrenade.cs");
   exec("./weapons/satchelCharge.cs");

   //gameplay types:
   exec("./flag.cs");
   exeC("./zoneControl.cs");
   exeC("./core.cs");

   exec("./health.cs");

   // Keep track of when the game started
   $Game::StartTime = $Sim::Time;
}

function onServerDestroyed()
{
   // This function is called as part of a server shutdown.
}


//-----------------------------------------------------------------------------

function onMissionLoaded()
{
   // Called by loadMission() once the mission is finished loading.
   // Nothing special for now, just start up the game play.
   startGame();
}

function onMissionEnded()
{
   // Called by endMission(), right before the mission is destroyed

   // Normally the game should be ended first before the next
   // mission is loaded, this is here in case loadMission has been
   // called directly.  The mission will be ended if the server
   // is destroyed, so we only need to cleanup here.
   cancel($Game::Schedule);
   $Game::Running = false;
   $Game::Cycling = false;
}


//-----------------------------------------------------------------------------

function startGame()
{
   if ($Game::Running) {
      error("startGame: End the game first!");
      return;
   }

   // Inform the client we're starting up
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameStart');

      // Other client specific setup..
      %cl.score = 0;

      %cl.clearStats();
   }

   // Start the game timer
   if ($Pref::Server::TimeLimit)
   {
      $Game::Schedule = schedule(($Pref::Server::TimeLimit * 60 * 1000) - (10 * 1000), 0, "onGameTenSecondsLeft" );
   }
   $Game::StartTime = getSimTime();
   $Game::Running = true;
}

function endGame()
{
   if (!$Game::Running)
   {
      error("endGame: No game running!");
      return;
   }

   // Stop any game timers
   cancel($Game::Schedule);

   // Inform the client the game is over
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ )
   {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameEnd');
      
      commandToClient(%cl, 'SetEndGameLine', " " TAB "Name" 
      TAB "Caps" 
      TAB "Stand_Grabs" 
      TAB "Field_Grabs" 
      
      TAB "Kills"
      TAB "Deaths"
      TAB "Self_Kills"
      TAB "Suicides"

      TAB "HKs_Used"
      TAB "Avg_HK_Waste"

      TAB "Discs_Fired"
      TAB "Direct_Hits"
      );

      for(%i = 0; %i < TeamManager_SO.numTeams; %i++)
      {  
         commandToClient(%cl, 'SetEndGameLine', "");
         commandToClient(%cl, 'SetEndGameLine', TeamManager_SO.teamScore[%i] TAB TeamManager_SO.teamName[%i]);

         for(%j = 0; %j < ClientGroup.getCount(); %j++)
         {
            %otherClient = ClientGroup.getObject(%j);
            if(%otherClient.team == %i)
            {
               //echo("client name = ", StripMLControlChars(getTaggedString(%otherclient.name)));
               if(mFloor(%otherClient.stat_HealthKits_Used) <= 0) 
                  %hkWaste = "N/A";
               else  
                  %hkWaste = (%otherClient.stat_HealthKit_Waste / mFloor(%otherClient.stat_HealthKits_Used));

               commandToClient(%cl, 'SetEndGameLine', "" 
                  TAB StripMLControlChars(getTaggedString(%otherclient.name))

                  TAB mFloor(%otherClient.stat_flag_captures)
                  TAB mFloor(%otherClient.stat_flag_standgrabs)
                  TAB mFloor(%otherClient.stat_flag_fieldgrabs)

                  TAB mFloor(%otherClient.stat_kills)
                  TAB mFloor(%otherClient.stat_deaths)
                  TAB mFloor(%otherClient.stat_self_kills)
                  TAB mFloor(%otherClient.stat_suicides)

                  TAB mFloor(%otherClient.stat_HealthKits_Used)
                  TAB %hkWaste

                  TAB mFloor(%otherClient.stat_RoundsFired_Disc)
                  TAB mFloor(%otherClient.stat_RoundsDirectHit_DiscLauncherProjectile)
                  );
            }
         }

     }

    
   }



   // Delete all the temporary mission objects
   resetMission();
   $Game::Running = false;

}

function onGameTenSecondsLeft()
{
   $Game::Schedule = schedule(10 * 1000, 0, "onGameDurationEnd" );

   //sync clock with all clients
   %count = ClientGroup.getCount();
   for(%i = 0; %i < %count; %i++)
   {
      %client = ClientGroup.getObject(%i);
      %client.syncClock(1);
   }
}

function onGameDurationEnd()
{
   // This "redirect" is here so that we can abort the game cycle if
   // the $Game::Duration variable has been cleared, without having
   // to have a function to cancel the schedule.
   if ($Game::Duration && !isObject(EditorGui))
      cycleGame();
}


//-----------------------------------------------------------------------------

function cycleGame()
{
   // This is setup as a schedule so that this function can be called
   // directly from object callbacks.  Object callbacks have to be
   // carefull about invoking server functions that could cause
   // their object to be deleted.
   if (!$Game::Cycling) {
      $Game::Cycling = true;
      $Game::Schedule = schedule(0, 0, "onCycleExec");
   }
}

function onCycleExec()
{
   // End the current game and start another one, we'll pause for a little
   // so the end game victory screen can be examined by the clients.
   endGame();
   $Game::Schedule = schedule($Game::EndGamePause * 1000, 0, "onCyclePauseEnd");
}

function onCyclePauseEnd()
{
   $Game::Cycling = false;

   // Just cycle through the missions for now.
   %search = $Server::MissionFileSpec;
   for (%file = findFirstFile(%search); %file !$= ""; %file = findNextFile(%search)) {
      if (%file $= $Server::MissionFile) {
         // Get the next one, back to the first if there
         // is no next.
         %file = findNextFile(%search);
         if (%file $= "")
           %file = findFirstFile(%search);
         break;
      }
   }
   loadMission(%file);
}


//-----------------------------------------------------------------------------
// GameConnection Methods
// These methods are extensions to the GameConnection class. Extending
// GameConnection make is easier to deal with some of this functionality,
// but these could also be implemented as stand-alone functions.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function GameConnection::onClientEnterGame(%this)
{
   %this.syncClock(30);

   // Create a new camera object.
   %this.camera = new Camera() {
      dataBlock = Observer;
   };
   MissionCleanup.add( %this.camera );
   %this.camera.scopeToClient(%this);

   messageClient(%this, 'InventoryReset');

   // Setup game parameters, the onConnect method currently starts
   // everyone with a 0 score.
   %this.score = 0;
   %this.team = 999;
   %this.playerClass = "Soldier";
   
   %this.setControlObject(%this.camera);

   %spawnPoint = pickSpawnPoint();
   %this.camera.setTransform( %spawnPoint );
   %this.camera.setMode("Observer");
   
   // Create a player object.
   //%this.spawnPlayer();
   TeamManager_SO.transmitTeams(%this);
}

function GameConnection::onClientLeaveGame(%this)
{
   if (isObject(%this.camera))
      %this.camera.delete();
   if (isObject(%this.player))
      %this.player.delete();

   //kill their "whos talking" entry
		serverCmdStopTalking(%this);

   TeamManager_SO.clientLeaveTeam(%this);
}


//-----------------------------------------------------------------------------

function GameConnection::onLeaveMissionArea(%client)
{
   // The control objects invoked this method when they
   // move out of the mission area.
   //echo("LEAVING MISSION AREA");

   //tell them that they're leaving the mission area
   //schedule countdown to death
   
   //%client.leftMissionArea();
}

function GameConnection::leftMissionArea(%client, %time)
{
   if(%time $= "")
      %time = 10;
   else
      %time--;

   if(%time <= 0)
   {
      messageClient(%client, '', '\c5BOOM');
      return;
   }
   
   messageClient(%client, '', '\c5RETURN TO THE MISSION AREA IN %1', %time);
   %client.missionAreaEvent = %client.schedule(1000, leftMissionArea, %time);
}

function GameConnection::onEnterMissionArea(%client)
{
   // The control objects invoked this method when they
   // move back into the mission area.
   //echo("ENTERING MISSION AREA");
   if(isEventPending(%client.missionAreaEvent))
      cancel(%client.missionAreaEvent);
}


//-----------------------------------------------------------------------------

function GameConnection::onDeath(%this, %sourceObject, %sourceClient, %damageType, %damLoc)
{
   // Clear out the name on the corpse
   %this.player.setShapeName("");

   %this.deathTime = getSimTime();
   %this.canRespawnCheck(1000);

   // Switch the client over to the death cam and unhook the player object.
   if (isObject(%this.camera) && isObject(%this.player)) {
      %this.camera.setMode("Corpse",%this.player);
      %this.setControlObject(%this.camera);
   }
   //%this.player = 0;

   // Doll out points and display an appropriate message
   if(%damageType $= "Suicide")
      %this.incStat("Suicides");
   else if(%sourceClient == %this || %damageType $= "Impact")
      %this.incStat("Self_Kills");
   else if(isObject(%sourceClient))
   {
      if(%sourcClient.team == %this.team)
         %sourceClient.incStat("TeamKills");
      else
         %sourceClient.incStat("Kills");
   }

   %this.incStat("Deaths");   

   %vicColor = trim(TeamManager_SO.getTeamHexColor(%this.team));
   %vicName = trim(stripMLControlChars(getTaggedString(%this.name)));

   if (%damageType $= "Suicide" || %sourceClient == %this) {
      %this.incScore(-1);
      //messageAll('MsgClientKilled','%1 %2 \c0takes his own life!',%vicColor,%vicName);

      messageAll('MsgClientKilled', "<spush><font:wingdings:20>\c7N<spop> " @ %vicColor @ %vicName);
   }
   else {
      %sourceClient.incScore(1);

      %sourceName = stripMLControlChars(getTaggedString(%sourceClient.name));
      %sourceColor=  TeamManager_SO.getTeamHexColor(%sourceClient.team);

      messageAll('MsgClientKilled', %sourceColor @ %sourceName @ " <spush><font:webdings:20>\c3~<spop> " @ %vicColor @ %vicName);

      //messageAll('MsgClientKilled','%1 gets nailed by %2!',%this.name,%sourceClient.name);

      //if (%sourceClient.score >= $Game::EndGameScore)
      //   cycleGame();
   }
}

//-----------------------------------------------------------------------------
// repeatedly check the client's corpse and tell them when they can respawn
function GameConnection::CanRespawnCheck(%client, %time)
{
   %player = %client.player;   
   %deadTime = getSimTime() - %client.deathTime;

   if(isEventPending(%client.CanRespawnCheckEvent))
      cancel(%client.CanRespawnCheckEvent);

   if(%deadTime < $Game::MinRespawnDelay)
   {
      %client.CanRespawnCheckEvent = %client.schedule(%time, CanRespawnCheck, %time);
      return;
   }
   else if (%deadTime > $Game::MaxRespawnDelay)
   {
      %canRespawn = true;
   }


   if(isObject(%player))
   {
      if(vectorLen(%client.player.getVelocity()) < $Game::MaxRespawnVelocity)
      {
         %start = %client.player.getWorldBoxCenteR();
         %end = vectorAdd(%start, "0 0 -10");
         %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType;
         %search = containerRayCast(%start, %end, %mask, %client.player);
         if(%search)
            %canRespawn = true;
      }
   }  
   else
   {
      %canRespawn = true;
   }

   if(%canRespawn)
      commandToClient(%client, 'CenterPrint', "Click To Respawn", -1);
   else
      %client.CanRespawnCheckEvent = %client.schedule(%time, CanRespawnCheck, %time);
}


//-----------------------------------------------------------------------------

function GameConnection::spawnPlayer(%client)
{
   //try to prevent duplicate players
   if(isObject(%client.player))
   {
      %client.player.delete();
      %client.player = "";
   }

   //if we've set a spawn team that is different than our current team, change now
   if(%client.spawnTeam != %client.team)
      if(%client.spawnTeam == -1) //auto-join
         TeamManager_SO.clientAutoJoinTeam(%client);
      else
         TeamManager_SO.clientJoinTeam(%client, %client.spawnTeam);

   //don't spawn if we're on the observer team
   if(%client.team != 999)
   {  
      // Combination create player and drop him somewhere
      %spawnPoint = TeamManager_SO.pickSpawnPoint(%client.team); //pickSpawnPoint();
      %client.createPlayer(%spawnPoint);
   }


   if(isEventPending(%client.CanRespawnCheckEvent))
      cancel(%client.CanRespawnCheckEvent);
   commandToClient(%client, 'ClearCenterPrint');
}   


//-----------------------------------------------------------------------------

function GameConnection::createPlayer(%this, %spawnPoint)
{
   if (%this.player > 0)  {
      // The client should not have a player currently
      // assigned.  Assigning a new one could result in 
      // a player ghost.
      error( "Attempting to create an angus ghost!" );
   }

   //hack: change datablock based on team number
   %data = %this.GetPlayerClassDataBlock();

   // Create the player object
   %player = new Player() {
      dataBlock = %data;
      client = %this;
   };
   MissionCleanup.add(%player);

   %player.team = %this.team;
   %player.spawnTime = getSimTime();

   // Player setup...
   %player.setTransform(%spawnPoint);
   %player.setEnergyLevel(%player.getDataBlock().maxEnergy);
   %player.setShapeName(%this.name);

   //echo("team color = ", TeamManager_SO.getTeamShapeNameColor(%this.team));
   //echo(" player = ", %player);
   %player.setShapeNameColor(TeamManager_SO.getTeamShapeNameColor(%this.team));
   
   // Update the camera to start with the player
   %this.camera.setTransform(%player.getEyeTransform());

   // Give the client control of the player
   %this.player = %player;
   %this.setControlObject(%player);

   //default inventory
   %this.GiveSpawnEquipment();

   //tell client about their inventory
   %this.TransmitInventory();

   %player.schedule($Game::SpawnFireDelay, setImageLoaded, 0, 1);
}


//-----------------------------------------------------------------------------
// Support functions
//-----------------------------------------------------------------------------

function pickSpawnPoint() 
{
   //radius + raycast down method from blockland
   %groupName = "MissionGroup/PlayerDropPoints";
   %group = nameToID(%groupName);

	if (%group != -1) {
		%count = %group.getCount();
		if (%count != 0) {
			%index = getRandom(%count-1);
			%spawn = %group.getObject(%index);
			//echo("spawn object = ", %spawn);
			%rayHeight = %spawn.RayHeight;
			if(%rayHeight $= "")
				%rayHeight = 10;

			%trans = %spawn.getTransform();
			
			%transX = getWord(%trans, 0);
			%transY = getWord(%trans, 1);
			%transZ = getWord(%trans, 2);
			
			for(%i = 0; %i < 1000; %i++)
			{
				%r = getRandom(%spawn.radius * 10) / 10;
				%ang = getRandom($pi * 2 * 100) / 100;
				
				//x = r * cos( theta )
				%transX = getWord(%trans, 0);
				%transY = getWord(%trans, 1);
				%transZ = getWord(%trans, 2);

				%transX += %r * mCos(%ang);
				%transY += %r * mSin(%ang);
				 
				//raycast from z val + ray height down
				%start = %transX SPC %transY SPC %transZ + %rayHeight;
				%end   = %transX SPC %transY SPC %transZ - 2;
				%mask = $TypeMasks::FxBrickObjectType | $TypeMasks::InteriorObjectType | $TypeMasks::StaticShapeObjectType | $TypeMasks::PlayerObjectType;

				%scanTarg = containerRayCast(%start,%end,%mask,0);
				//error("scan targ = ", %scanTarg);
				
				if(%scanTarg)
				{
					%scanPos = posFromRaycast(%scanTarg);
					%transZ = getWord(%scanPos, 2);
					//error("scan pos = ", %scanPos);

					%boxCenter = vectorAdd(%scanPos, "0 0 1.6");
					%mask = $TypeMasks::FxBrickObjectType | $TypeMasks::StaticShapeObjectType | $TypeMasks::PlayerObjectType; 
					if(containerBoxClear(%mask, %boxCenter, 0.6, 0.6, 1.3))
					{
						//warn("Box cleared!");
						break;
					}
				}				
			}		
			

			//%transZ = getTerrainHeight(%transXY);
         
         if(%spawn.directional)
            %spawnAngle = " " @ getWords(%spawn.getTransform(), 3, 6);
         else
   			%spawnAngle = " 0 0 1 " @ getRandom($pi * 2 * 100) / 100;

			%returnTrans = %transX  @ " " @ %transY @ " " @ %transZ @ %spawnAngle;
			
			return %returnTrans;
			//return %spawn.getTransform();
		}
		else
			error("No spawn points found in " @ %groupName);
	}
	else
	error("Missing spawn points group " @ %groupName);

	error("default spawn!");
   // Could be no spawn points, in which case we'll stick the
   // player at the center of the world.
   return "0 0 300 1 0 0 0";

   
// VVV Default method (sucks) VVV

//   %groupName = "MissionGroup/PlayerDropPoints";
//   %group = nameToID(%groupName);
//
//   if (%group != -1) {
//      %count = %group.getCount();
//      if (%count != 0) {
//         %index = getRandom(%count-1);
//         %spawn = %group.getObject(%index);
//         return %spawn.getTransform();
//      }
//      else
//         error("No spawn points found in " @ %groupName);
//   }
//   else
//      error("Missing spawn points group " @ %groupName);
//
//   // Could be no spawn points, in which case we'll stick the
//   // player at the center of the world.
//   return "0 0 300 1 0 0 0";
}


function GameConnection::incStat(%client, %statName, %amt)
{
   if(%amt $= "")
      %amt = 1;
   
   //echo("increasing stat ", %statname , " by ", %amt);
   %client.stat_[%statName] += %amt;
}

function GameConnection::dumpStats(%client)
{
   %count = %client.getDynamicFieldCount();
   
   echo("");
   for(%i = 0; %i < %count; %i++)
   {
      %dynField = %client.getDynamicField(%i);

      %name = getField(%dynField, 0);
      %value = getField(%dynField, 1);


      %sub = getSubStr(%name, 0, 5);
      if(%sub $= "stat_")
      {
         %statName = getSubStr(%name, 5, strLen(%name) - 5);
         echo("  "  @ %statName @ " : " @ %value);
      }
   }
   echo("");
}

function GameConnection::clearStats(%client)
{
   %count = %client.getDynamicFieldCount();
  
   for(%i = 0; %i < %count; %i++)
   {
      %dynField = %client.getDynamicField(%i);

      %name = getField(%dynField, 0);
      %value = getField(%dynField, 1);

      %sub = getSubStr(%name, 0, 5);
      %rest = getSubStr(%name, 5, strLen(%name) - 5);
      //echo(" sub = ", %sub);
      //echo("  rest = ", %rest);
      if(%sub $= "stat_")
      {
         %client.stat_[%rest] = 0;
      }
   }
}

function GameConnection::syncClock(%client, %frequency)
{
   if(isEventPending(%client.syncClockEvent))
      cancel(%client.syncClockEvent);

   if(!%frequency)
      %frequency = 30;
   
   %gameTimeLeft = ($Pref::Server::TimeLimit * 60 * 1000) - (getSimTime() - $Game::StartTime);
   commandToClient(%client, 'SyncClock', %gameTimeLeft);

   %client.syncClockEvent = %client.schedule(%frequency * 1000, syncClock, %frequency);
}

function GameConnection::getColoredName(%client)
{
   %name = StripMLControlChars(getTaggedString(%client.name));
   %color = TeamManager_SO.getTeamHexColor(%client.team);

   //echo(" name = ", %name);
   //echo(" color = ", %color);

   return(%color @ %name);
}