//inventory.cs

function Player::IncInventory(%player, %itemData, %amount, %doNotTransmit)
{
   //echo("inc inventory ", %player, "|", %itemData, "|", %amount);
  
   %curr = %player.getInventory(%itemData);
   %max = %player.getMaxInventory(%itemData);
   
   if(%max - %curr < %amount)
      %amount = %max - %curr;

   if(%amount <= 0)
      return 0;

   //enforce item category restrictions
   %class = %itemData.className;
   if(%class !$= "")
   {
      if(%player.getDataBlock().maxInvClass[%class] !$= "")
      {
         if(%player.invClass[%class] >= %player.getDataBlock().maxInvClass[%class])
            return 0;
         
         %player.invClass[%class]++;
      }
   }

   //increase inventory
   %player.inv[%itemData.getName()] += %amount;
   //call back to item
   %itemData.onInventory(%player,%player.inv[%itemData.getName()]);

   //tell client 
   %client = %player.client;
   if( !isObject(%client) || (%player.getState() $= "Dead") )
      return %amount;
  
   if(%doNotTransmit)
      return %amount;

   if(%itemData.category $= "Ammo")
      messageClient(%client, 'InventoryAmount', '', %itemData.getID(), %player.inv[%itemData.getName()], "", %itemData.weapon.getID());
   else
      messageClient(%client, 'InventoryAmount', '', %itemData.getID(), %player.inv[%itemData.getName()]);

   return %amount;
}

function Player::DecInventory(%player, %itemData, %amount)
{
   %curr = %player.getInventory(%itemData);
   
   if(%amount > %curr)
      %amount = %curr;

   if(%amount <= 0)
      return false;
   
   //enforce item category restrictions
   %class = %itemData.className;
   if(%class !$= "")
      %player.invClass[%class]--;

   //decrease inventory
   %player.inv[%itemData.getName()] -= %amount;
   //call back to item
   %itemData.onInventory(%player,%player.inv[%itemData.getName()]);

   //tell client 
   %client = %player.client;
   if(!isObject(%client)|| (%player.getState() $= "Dead") )
      return true;

   if(%itemData.category $= "Ammo")
      messageClient(%client, 'InventoryAmount', '', %itemData.getID(), %player.inv[%itemData.getName()], "", %itemData.weapon.getID());
   else
      messageClient(%client, 'InventoryAmount', '', %itemData.getID(), %player.inv[%itemData.getName()]);

   return true;
}

function Player::GetInventory(%player, %itemData)
{
   return mFloor(%player.inv[%itemData.getName()]);
}
function Player::GetMaxInventory(%player, %itemData)
{
   return mFloor(%player.getDataBlock().maxInv[%itemData.getName()]);
}


function Player::DumpInventory(%player)
{
   //cycle through all item datablocks and print out how many we have of each
   echo("");
   echo("Player " @ %player @ "'s (" @ getTaggedString(%player.client.name) @ ") Inventory: ");
   %count = DataBlockGroup.getCount();
   for(%i = 0; %i < %count; %i++)
   {
      %data = DataBlockGroup.getObject(%i);

      if(%data.getClassName() !$= "ItemData")
         continue;
      
      %currInv = %player.getInventory(%data);
      %maxInv = %player.getMaxInventory(%data);
      if(%currInv != 0 || %maxInv != 0)
      {
         if(%currInv > %maxInv)
            error("  " @ %data.getName() @ ": " @ %currInv @ "/" @ %maxInv);   
         else
            echo("  " @ %data.getName() @ ": " @ %currInv @ "/" @ %maxInv);
      }
   }

   echo("");
}


function GameConnection::TransmitInventory(%client)
{
   %player = %client.player;
   
   if(!isObject(%player))
      return;
   
   messageClient(%client, 'InventoryReset');

   %count = DataBlockGroup.getCount();
   for(%i = 0; %i < %count; %i++)
   {
      %data = DataBlockGroup.getObject(%i);

      if(%data.getClassName() !$= "ItemData")
         continue;
      
      %currInv = %player.getInventory(%data);
      %maxInv = %player.getMaxInventory(%data);

      if(%maxInv != 0)
         messageClient(%client, 'InventoryAmount', '',  %data.getID(), %currInv, %maxInv);
   }

   messageClient(%client, 'InventoryDone');
}



function Player::Throw(%player, %itemData, %amt)
{
   %invAmt = %player.getInventory(%itemData);
   if(%invAmt <=0)
      return false;

   if(%amt <= 0)
      %amt = mFloor(%itemData.maxInventory);
   if(%amt <= 0)
      %amt = 1;

   if(%invAmt < %amt)
      %amt = %invAmt;
     
   %thrownItem = new Item()
   {
      datablock = %itemData.getID();
      rotation = "0 0 1 " @ (getRandom() * 360);
      count = %amt;
   };
   MissionCleanup.add(%thrownItem);
   %thrownItem.schedulePop();
   %thrownItem.setCollisionTimeout(%player);
   
   %thrownItem.setTransform(%player.getWorldBoxCenter());

   %vector = %player.getEyeVector();
   %vector = vectorAdd(%vector, "0 0 0.5");
   %vector = vectorNormalize(%vector);
   %vector = vectorScale(%vector, 15);
   %vector = vectorAdd(%vector, %player.getVelocity());
   %thrownItem.setVelocity(%vector);

   %player.decInventory(%itemData, %amt);

   return %thrownItem;
}
