//LaserRifle.cs

datablock ItemData( LaserRifle )
{
    category             = "Weapon";
    className            = "Weapon";
    
    // -- dont pay any attention to this variable UNLESS this is a mod weapon
    //    this value associates a weapon icon and reticle with this weapon.
    //    note that the BASE client only contains associations for base weapons.
    //    this means that server side mods should only use profiles for base weapons.
    clientProfile        = "LaserRifleLauncher";
    
    shapeFile            = "~/data/shapes/weapons/disc_proj.dts";
    mass                 = 100;
    elasticity           = 0.2;
    friction             = 0.6;
    emap                 = true;

    inventory            = true;
    inventoryName        = "Laser Rifle";
    hudName              = "Laser";
    inventoryRequirement = "";
    inventoryDescription = "";
    iconName = "laserRifle";
    
    computeCRC           = $pref::Server::checkCRC;

    throwForce           = 1500;

    image                = LaserRifleImage;
    ammo                 = "";
};

// -----------------------------------------------------------------------------

datablock ShapeBaseImageData( LaserRifleImage )
{
    className                        = "WeaponImage";
    shapeFile                        = "~/data/shapes/weapons/disc_proj.dts";
    mountPoint                       = $WeaponSlot;

    //                                 +R/-L | +F/-B | +U/-D
    //offset                           = "0.15 -0.40 0.00";
    eyeOffset                        = "0.25 0.43 -0.34";
    rotation                         = "0 0 1 180";
    eyeRotation                      = "0 0 1 180";
    scale = "0.75 0.25 0.75";

    usesEnergy                       = true;
    minEnergy                        = 4;

    lightType                        = "NoLight";
    lightColor                       = "1.000000 1.000000 1.000000 1.000000";
    lightTime                        = "1000";
    lightRadius                      = "10";

    shellExitDir                     = "0.707107 0 0.707107";
    shellExitVariance                = "20";
    shellVelocity                    = "5";

    cloakable                        = true;
    correctMuzzleVector              = true;
    emap                             = true;
    firstPerson                      = true;

    projectileType                   = LinearProjectile;  
    item                             = LaserRifle;
    casing                           = DiscLauncherShell;
    ammo                             = "";
    projectile                       = LaserAProjectile;
    projectileSpread                 = 0/1000;
    
    // ----

    stateName[0]                     = "PreActivate";
    stateTransitionOnLoaded[0]       = "Ready";

    stateName[1]                     = "Ready";
    stateTransitionOnNoAmmo[1]       = "Noammo";
    stateTransitionOnTriggerDown[1]  = "Fire";
    stateSequence[1]                 = "ready";
    stateSound[1]                    = "DiscLauncherIdleSound";

    stateName[3]                     = "Fire";
    stateTransitionOnTimeout[3]      = "Reload";
    stateTimeoutValue[3]             = 0.1;
    stateFire[3]                     = true;
    //stateRecoil[3]                   = LightRecoil;
    stateAllowImageChange[3]         = false;
    stateSequence[3]                 = "Fire";
    stateScript[3]                   = "onFire";
    stateEjectShell[3]               = true;
    stateEnergyDrain[3] = 9999;
    stateSound[3]                    = "DiscLauncherFireSound";

    stateName[4]                     = "Reload";
    stateTransitionOnAmmo[4]         = "doReload";
    stateTransitionOnNoAmmo[4]       = "NoAmmo";
    stateAllowImageChange[4]         = false;

    stateName[5]                     = "Noammo";
    stateTransitionOnAmmo[5]         = "doReload";
    stateSequence[5]                 = "Noammo";
    stateTransitionOnTriggerDown[5]  = "DryFire";

    stateName[6]                     = "Dryfire";
    stateSound[6]                    = "DiscLauncherDryfireSound";
    stateTimeoutValue[6]             = 1.0;
    stateSequence[6]                 = "Noammo";
    stateTransitionOnTimeout[6]      = "Noammo";

    stateName[7]                     = "CheckAmmo";
    stateTransitionOnAmmo[7]         = "Ready"; //"Activate";
    stateTransitionOnNoAmmo[7]       = "NoAmmo";
  

    stateName[8]                     = "doReload";
    stateTransitionOnTimeout[8]      = "Ready";
    stateTimeoutValue[8]             = 0.725;
    stateSequence[8]                 = "Reload";
    stateSound[8]                    = "DiscLauncherReloadSound";
    stateAllowImageChange[8]         = false;
};


function LaserRifleImage::onFire(%data, %obj, %slot)
{
   //echo("laser rifle fire ", %data, "|", %obj, "|", %slot);

   //echo("  energy level ", %obj.getEnergyLevel());
   %energyLevel = %obj.getEnergyLevel();
   %damageLevel = %energyLevel / %obj.getDataBlock().maxEnergy;

   //echo("  damage level = ", %damageLevel);

   if(%damageLevel >= 0.9)
      %projectileData = LaserAProjectile;
   else if(%damageLevel >= 0.5)
      %projectileData = LaserBProjectile;
   else
      %projectileData = LaserCProjectile;

   //%projectileData = LaserCProjectile;

   %obj.client.incStat("RoundsFired_" @ %data.item.hudName);

   %start = %obj.getEyePoint();
   %vec = %obj.getEyeVector();
   %end = vectorAdd(%start, vectorScale(%vec, 1000));
   %mask = $TypeMasks::PlayerObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::StaticObjectType;

   %search = ContainerRayCast(%start, %end, %mask, %obj);

   if(%search)
   {
      %victimObj = getWord(%search, 0);
      %pos = getWords(%search, 1, 3);
      %pos = vectorAdd(%pos, vectorScale(%vec, -0.1) );
      %normal = getWords(%search, 4,6);

      %p = new (linearProjectile)()
      {
         dataBlock = %projectileData;
         initialVelocity = vectorScale(%vec, 2000);
         initialPosition = %pos;
         sourceObject = %obj;
         sourceSlot = %slot;
         client = %obj.client;
      };
      MissionCleanup.add(%p);
      %p.damageLevel = %damageLevel;
   }
   

}