
// -----------------------------------------------------------------------------
// -- Logical, Projectile and Weapon
// -----------------------------------------------------------------------------

datablock LinearProjectileData( DiscLauncherProjectile )
{
    projectileShapeName  = "~/data/shapes/weapons/disc_proj.dts";
    scale                = "2.4 2.4 2.4";

    sound                = DiscLauncherProjectileSound;
    explosion            = groundMinorExplosion;
//  particleEmitter      = DiscLauncherTrailEmitter;

    directDamage         = 0;
    radiusDamage         = 60;
    damageRadius         = 9;
    damageType           = $DamageType::Disc;
    impulseForce         = 1500;
    impulseVertical      = 500;
    impulseRadius        = 10;


    muzzleVelocity       = 163;//133; //123
    velInheritFactor     = 0.75;

    lifetime             = 6500;
    fadeDelay            = 1500;

    hasLight             = true;
    lightRadius          = 5.0;
    lightColor           = "1 0.4 0.3";
};

datablock DebrisData( DiscLauncherShell )
{
    shapeFile             = "~/data/shapes/weapons/disc_shell.dts";

    lifetime              = 5.0;
    minSpinSpeed          = 300.0;
    maxSpinSpeed          = 400.0;

    elasticity            = 0.7;
    friction              = 0.2;
    numBounces            = 3;

    staticOnMaxBounce     = true;
    snapOnMaxBounce       = false;
    fade                  = true;
};

// -----------------------------------------------------------------------------
// -- Ammunition
// -----------------------------------------------------------------------------

datablock ItemData( DiscLauncherAmmo )
{
    category         = "Ammo";
    className        = "Ammo";

    shapeFile        = "~/data/shapes/weapons/disc_shell.dts";
    mass             = 80;
    elasticity       = 0.2;
    friction         = 0.6;

    throwForce       = 1000;

    //pickUpName       = "disc launcher ammo";
    inventoryName    = "Disc Launcher Ammo";
    maxInventory     = 5;
    weapon           = DiscLauncher;
    regendelay       = 3000;
    energyUsage      = 15;
};

// -----------------------------------------------------------------------------

datablock ItemData( DiscLauncher )
{
    category             = "Weapon";
    className            = "Weapon";
    
    // -- dont pay any attention to this variable UNLESS this is a mod weapon
    //    this value associates a weapon icon and reticle with this weapon.
    //    note that the BASE client only contains associations for base weapons.
    //    this means that server side mods should only use profiles for base weapons.
    clientProfile        = "DiscLauncher";
    
    shapeFile            = "~/data/shapes/weapons/disc_wep.dts";
    mass                 = 100;
    elasticity           = 0.2;
    friction             = 0.6;
    emap                 = true;

    inventory            = true;
    inventoryName        = "Disc Launcher";
    hudName              = "Disc";
    inventoryRequirement = "";
    inventoryDescription = "";
    iconName = "discLauncher";

    
    computeCRC           = $pref::Server::checkCRC;

    throwForce           = 1500;

    //pickUpName           = "a disc launcher";
    image                = DiscLauncherImage;
    ammo                 = DiscLauncherAmmo;
};

// -----------------------------------------------------------------------------

datablock ShapeBaseImageData( DiscLauncherImage )
{
    className                        = "WeaponImage";
    shapeFile                        = "~/data/shapes/weapons/disc_wep.dts";
    mountPoint                       = $WeaponSlot;

    //                                 +R/-L | +F/-B | +U/-D
    //offset                           = "0.15 -0.40 0.00";
    eyeOffset                        = "0.25 0.43 -0.34";
    rotation                         = "1 0 0 0";
    eyeRotation                      = "1 0 0 0";
    //scale = "0.5 0.5 0.5";

    usesEnergy                       = false;
    minEnergy                        = 0;

    lightType                        = "NoLight";
    lightColor                       = "1.000000 1.000000 1.000000 1.000000";
    lightTime                        = "1000";
    lightRadius                      = "10";

    shellExitDir                     = "0.707107 0 0.707107";
    shellExitVariance                = "20";
    shellVelocity                    = "5";

    cloakable                        = true;
    correctMuzzleVector              = true;
    emap                             = true;
    firstPerson                      = true;

    projectileType                   = LinearProjectile;  
    item                             = DiscLauncher;
    casing                           = DiscLauncherShell;
    ammo                             = DiscLauncherAmmo;
    projectile                       = DiscLauncherProjectile;
    projectileSpread                 = 0/1000;
    
    // ----

    stateName[0]                     = "PreActivate";
    stateTransitionOnLoaded[0]       = "CheckAmmo";

    stateName[1]                     = "Activate";
    stateTransitionOnTimeout[1]      = "Ready";
    stateTimeoutValue[1]             = 0.725;
    stateSequence[1]                 = "ready";
    stateSound[1]                    = "DiscLauncherMountSound";

    stateName[2]                     = "Ready";
    stateTransitionOnNoAmmo[2]       = "Noammo";
    stateTransitionOnTriggerDown[2]  = "Fire";
    stateSequence[2]                 = "ready";
    stateSound[2]                    = "DiscLauncherIdleSound";

    stateName[3]                     = "Fire";
    stateTransitionOnTimeout[3]      = "Hold";
    stateTimeoutValue[3]             = 0.2;
    stateFire[3]                     = true;
    stateRecoil[3]                   = LightRecoil;
    stateAllowImageChange[3]         = false;
    stateSequence[3]                 = "Fire";
    stateScript[3]                   = "onFire";
    stateEjectShell[3]               = true;
    stateSound[3]                    = "DiscLauncherFireSound";
    stateTransitionOnTriggerUp[3]   = "Reload";

    stateName[9] = "Hold";
    stateScript[9] = "onHold";
    stateTransitionOnTriggerUp[9] = "Release";

    stateName[10] = "Release";
    stateScript[10] = "onRelease";
    stateTimeoutValue[10] = 0.1;
    stateTransitionOnTimeout[10] = "Reload";

    stateName[4]                     = "Reload";
    stateTransitionOnAmmo[4]         = "doReload";
    stateTransitionOnNoAmmo[4]       = "NoAmmo";
    stateAllowImageChange[4]         = false;

    stateName[5]                     = "Noammo";
    stateTransitionOnAmmo[5]         = "doReload";
    stateSequence[5]                 = "Noammo";
    stateTransitionOnTriggerDown[5]  = "DryFire";

    stateName[6]                     = "Dryfire";
    stateSound[6]                    = "DiscLauncherDryfireSound";
    stateTimeoutValue[6]             = 1.0;
    stateSequence[6]                 = "Noammo";
    stateTransitionOnTimeout[6]      = "Noammo";

    stateName[7]                     = "CheckAmmo";
    stateTransitionOnAmmo[7]         = "Ready"; //"Activate";
    stateTransitionOnNoAmmo[7]       = "NoAmmo";
  

    stateName[8]                     = "doReload";
    stateTransitionOnTimeout[8]      = "Ready";
    stateTimeoutValue[8]             = 0.725;
    stateSequence[8]                 = "Reload";
    stateSound[8]                    = "DiscLauncherReloadSound";
    stateAllowImageChange[8]         = false;
};

// -----------------------------------------------------------------------------
// -- Functions
// -----------------------------------------------------------------------------
function DiscLauncherImage::onFire(%data, %player, %slot)
{
   %p = Parent::OnFire(%data, %player, %slot);
   %player.projectile = %p;
}

function DiscLauncherImage::onHold(%data, %player, %slot)
{

}

function DiscLauncherImage::onRelease(%data, %player, %slot)
{
   if(isObject(%player.projectile))
      %player.projectile.explode();
}


// -----------------------------------------------------------------------------
