// Projectile Particle Trail
datablock ParticleData(MortarTrailParticle)
{
   dragCoeffiecient     = 0.0;
   gravityCoefficient   = -0.2;   // rises slowly
   inheritedVelFactor   = 0.05;

   lifetimeMS           = 700;  // lasts 2 second
   lifetimeVarianceMS   = 150;   // ...more or less

   textureName          = "~/data/textures/effects/particle/smoke02";

   useInvAlpha = true;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0]     = "0.0 0.9 0.0 1.0";
   colors[1]     = "0.0 0.6 0.0 1.0";
   colors[2]     = "0.0 0.4 0.0 0.0";

   sizes[0]      = 0.5;
   sizes[1]      = 1.5;
   sizes[2]      = 3.5;

   times[0]      = 0.0;
   times[1]      = 0.2;
   times[2]      = 1.0;

};

datablock ParticleEmitterData(MortarTrailEmitter)
{
  ejectionPeriodMS = 5;
   periodVarianceMS = 5;

   ejectionVelocity = 1.25;
   velocityVariance = 0.50;

   thetaMin         = 0.0;
   thetaMax         = 90.0;  

   particles = "MortarTrailParticle";
};


// -----------------------------------------------------------------------------
// -- Logical, Projectile and Weapon
// -----------------------------------------------------------------------------

datablock GrenadeProjectileData( MortarProjectile )
{
    projectileShapeName  = "~/data/shapes/weapons/disc_shell.dts";
    scale                = "2 2 2";

    sound                = MortarProjectileSound;
    explosion            = GroundMajorExplosion;
    emitter              = MortarTrailEmitter;

    directDamage         = 5;
    radiusDamage         = 175;
    damageRadius         = 18;
    damageType           = $DamageType::Mortar;

    impulseForce         = 2000;
    impulseVertical      = 1500;
    impulseRadius        = 16;
    impactImpulse        = 300;
    impactImpulseVertical = 300;

    muzzleVelocity       = 65; //140
    velInheritFactor     = 1.0;  //.2

    lifetime             = 25000;
    fadeDelay            = 1500;
    armingDelay          = 1500; //1600

    elasticity           = 0.2; //.20
    friction             = 0.6;
    drag                 = 0.0;
    gravityMod           = (20 / 9.81);
    isBallistic          = true;

    hasLight             = true;
    lightRadius          = 5.0;
    lightColor           = "0.6 0.3 0";

    scaleDirectDamageBySpeed = true;
};

datablock DebrisData( MortarShell )
{
    shapeFile             = "~/data/shapes/weapons/mortar_shell.dts";

    lifetime              = 5.0;
    minSpinSpeed          = 200.0;
    maxSpinSpeed          = 300.0;

    elasticity            = 0.4;
    friction              = 0.0;
    numBounces            = 2;

    staticOnMaxBounce     = true;
    snapOnMaxBounce       = false;
    fade                  = true;
    emap             = true;
};

// -----------------------------------------------------------------------------
// -- Ammunition
// -----------------------------------------------------------------------------

datablock ItemData( MortarAmmo )
{
    category         = "Ammo";
    className        = "Ammo";

    shapeFile        = "~/data/shapes/weapons/mortar_ammo.dts";
    mass             = 100;
    elasticity       = 0.2;
    friction         = 0.6;

    throwForce           = 1000;

    InventoryName = "Mortar Ammo ";
    maxInventory     = 4;
    weapon           = Mortar;
    regendelay       = 10000;
    energyUsage      = 25;
};

datablock ItemData( Mortar )
{
    category             = "Weapon";
    className            = "Weapon";

    // -- dont pay any attention to this variable UNLESS this is a mod weapon
    //    this value associates a weapon icon and reticle with this weapon.
    //    note that the BASE client only contains associations for base weapons.
    //    this means that server side mods should only use profiles for base weapons.
    clientProfile        = "DiscLauncher";

    shapeFile            = "~/data/shapes/weapons/mortar_wep.dts";
    mass                 = 150;
    elasticity           = 0.2;
    friction             = 0.6;
    emap                 = true;

    inventory            = true;
    inventoryName        = "Mortar";
    hudName				    = "Mortar";
    inventoryRequirement = "";
    inventoryDescription = "";
    iconName = "mortar";

    computeCRC           = $pref::Server::checkCRC;

    throwForce           = 1500;

    pickUpName           = "a mortar launcher";
    image                = MortarImage;
    ammo                 = MortarAmmo;
};

datablock ShapeBaseImageData(MortarImage)
{
    className                        = "WeaponImage";
    shapeFile                        = "~/data/shapes/weapons/mortar_wep.dts";
    mountPoint                       = $WeaponSlot;

    //                                 +R/-L | +F/-B | +U/-D
    //offset                           = "0.1 -0.1 -0.1";
    eyeOffset                        = "0.5 1.0 -0.7"; //0.9 1.3 -1.4
    rotation                         = "1 0 0 0";
    eyeRotation                      = "1 0 0 0";

    usesEnergy                       = false;
    minEnergy                        = 0;

    lightType                        = "NoLight";
    lightColor                       = "1.000000 1.000000 1.000000 1.000000";
    lightTime                        = "1000";
    lightRadius                      = "10";

    shellExitDir                     = "0.707107 0 0.707107";
    shellExitVariance                = "20";
    shellVelocity                    = "2";

    cloakable                        = true;
    correctMuzzleVector              = true;
    emap                             = true;
    firstPerson                      = true;

//    projectileType                   = GrenadeProjectile;// old 0.4.1b method
    projectileType                   = GrenadeProjectile;  // for 0.4.2b
    item                             = Mortar;
    casing                           = MortarShell;
    ammo                             = MortarAmmo;
    projectile                       = MortarProjectile;
    projectileSpread                 = 0/1000;

    // ----

    // -- Initial start up state
    stateName[0] = "Preactivate";
    stateTransitionOnLoaded[0] = "Activate";
    stateTransitionOnNoAmmo[0] = "NoAmmo";


    // -- Activating the gun.Called when the weapon is first
    //    mounted and there is ammo.
    stateName[1] = "Activate";
    stateSound[1] = "MortarMountSound";
    stateTransitionOnTimeout[1] = "Ready";
    stateTimeoutValue[1] = 0.05;
    //stateSound[1] = "MortarIdleSound";


    // -- Ready to fire when triggered.
    stateName[2] = "Ready";
    stateTransitionOnTriggerDown[2] = "Fire";
    stateTransitionOnNoAmmo[2] = "NoAmmo";

    // -- when ready gets trigger hop in here and fire a bullet
    stateName[3] = "Fire";
    stateSequence[3] = "Fire";
    stateTimeoutValue[3] = 1.6; //1.4
    //stateTimeoutValue[3] = 0.5;
    stateRecoil[3] = LightRecoil;
    stateAllowImageChange[3] = false;
    stateFire[3] = true;
    stateEjectShell[3] = true;
    stateScript[3] = "onFire";
    stateTransitionOnTimeout[3] = "Reload";
    stateSound[3] = MortarFireSound;

    stateName[4] = "Reload";
    stateTimeoutValue[4] = 0.6;  //1.0
    stateTransitionOnTimeout[4] = "Ready";
    stateTransitionOnNoAmmo[4] = "NoAmmo";
    stateSequence[4] = "Reload";
    stateAllowImageChange[4] = false;
    stateSound[4] = MortarReloadSound;

    stateName[5] = "NoAmmo";
    stateTransitionOnAmmo[5] = "Reload";
    stateSequence[5] = "NoAmmo";
    stateTransitionOnTriggerDown[5]= "DryFire";

    // -- No ammo dry fire
    stateName[6] = "DryFire";
    stateTimeoutValue[6] = 1.4;
    stateTransitionOnTimeout[6] = "NoAmmo";
    stateSound[6] = MortarDryFireSound;
};


// -----------------------------------------------------------------------------
// -- Functions
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
