//turretGun.cs

//gun that shoots deployable turrets

datablock StaticShapeData(Turret)
{
	category = "Deployable";   // Mission editor category

	className = "Turret";      
	shapeFile = "~/data/shapes/items/flag_wht.dts";
	eMap = false;	
		
	isShielded = true;
	energyPerDamagePoint = 110;
	maxEnergy = 50;
	rechargeRate = 0.20;
	renderWhenDestroyed = false;
	doesRepair = true;

   //deployable stuff
   dynamicType    = $TypeMasks::StationObjectType;
	deployedObject = true;
   explosionSound = glassExplosionSound;
   
   maxDamage      = 100;
	disabledLevel  = 80;
	explosion      = glassExplosion;
	expDmgRadius   = 8.0;
	expDamage      = 0.35;
	expImpulse     = 500.0;

   //placement constraints
   maxSlope       = 360;    // maximum slope you can deploy on
   alignToNormal  = true;  //
   playerClearRadius = 1;  // cannot be any players within this radius
   staticClearRadius = 0.2;     // 


   //turret stuff
   scanTime    = 200;
   scanRadius  = 75;
   projectileVelocity = 120;
   projectile         = ChaingunBullet;
   projectileType     = LinearProjectile;
};

$Game::DeployableLimit_Player[Turret.getID()] = 5;
$Game::DeployableLimit_Team[Turret.getID()] = 5;

function Turret::onAdd(%this, %obj)
{
   echo("turret onadd", %this, "|", %obj);
}

// -----------------------------------------------------------------------------
// -- Ammunition
// -----------------------------------------------------------------------------

datablock ItemData( TurretGunAmmo )
{
    category         = "Ammo";
    className        = "Ammo";

    shapeFile        = "~/data/shapes/weapons/disc_shell.dts";
    mass             = 80;
    elasticity       = 0.2;
    friction         = 0.6;

    throwForce       = 1000;

    //pickUpName       = "disc launcher ammo";
    inventoryName    = "Turret Ammo";
    maxInventory     = 5;
    weapon           = TurretGun;
    regendelay       = 3000;
    energyUsage      = 15;
};

datablock ParticleData(TurretGunBeamParticle)
{
	dragCoefficient      = 0;
	gravityCoefficient   = 0;
	inheritedVelFactor   = 0.0;
	constantAcceleration = 0.0;
	lifetimeMS           = 50;
	lifetimeVarianceMS   = 0;
	textureName          = "~/data/textures/effects/particle/spark01";
	spinSpeed		= 0.0;
	spinRandomMin		= -0.0;
	spinRandomMax		= 0.0;
	colors[0]     = "0.2 0.2 1 1.0";
	colors[1]     = "0.2 0.2 1 0.0";
	sizes[0]      = 0.2;
	sizes[1]      = 0.2;
};
datablock ParticleEmitterData(TurretGunBeamEmitter)
{
   ejectionPeriodMS = 1;
   periodVarianceMS = 0;
   ejectionVelocity = 100;
   velocityVariance = 0.0;
   ejectionOffset   = -2;
   ejectionOffsetVariance = 0.0;
   thetaMin         = 0;
   thetaMax         = 0;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   particles = "TurretGunBeamParticle";
};


datablock ItemData( TurretGun )
{
    category             = "Weapon";
    className            = "Weapon";
    
    // -- dont pay any attention to this variable UNLESS this is a mod weapon
    //    this value associates a weapon icon and reticle with this weapon.
    //    note that the BASE client only contains associations for base weapons.
    //    this means that server side mods should only use profiles for base weapons.
    clientProfile        = "TurretGun";
    
    shapeFile            = "~/data/shapes/weapons/disc_wep.dts";
    mass                 = 100;
    elasticity           = 0.2;
    friction             = 0.6;
    emap                 = true;

    inventory            = true;
    inventoryName        = "Turret Gun";
    hudName              = "Turret Gun";
    inventoryRequirement = "";
    inventoryDescription = "";
    iconName = "TurretGun";

    
    computeCRC           = $pref::Server::checkCRC;

    throwForce           = 1500;

    //pickUpName           = "a disc launcher";
    image                = TurretGunImage;
    ammo                 = TurretGunAmmo;
};

// -----------------------------------------------------------------------------

datablock ShapeBaseImageData( TurretGunImage )
{
    className                        = "WeaponImage";
    shapeFile                        = "~/data/shapes/weapons/disc_wep.dts";
    mountPoint                       = $WeaponSlot;

    //                                 +R/-L | +F/-B | +U/-D
    //offset                           = "0.15 -0.40 0.00";
    eyeOffset                        = "0.25 0.43 -0.34";
    rotation                         = "1 0 0 0";
    eyeRotation                      = "1 0 0 0";
    //scale = "0.5 0.5 0.5";

    usesEnergy                       = false;
    minEnergy                        = 0;

    lightType                        = "NoLight";
    lightColor                       = "1.000000 1.000000 1.000000 1.000000";
    lightTime                        = "1000";
    lightRadius                      = "10";

    shellExitDir                     = "0.707107 0 0.707107";
    shellExitVariance                = "20";
    shellVelocity                    = "5";

    cloakable                        = true;
    correctMuzzleVector              = true;
    emap                             = true;
    firstPerson                      = true;

    projectileType                   = LinearProjectile;  
    item                             = TurretGun;
    casing                           = "";
    ammo                             = TurretGunAmmo;
    projectile                       = "";
    projectileSpread                 = 0/1000;
    
    // ----

    stateName[0]                     = "PreActivate";
    stateTransitionOnLoaded[0]       = "CheckAmmo";

    stateName[1]                     = "Activate";
    stateTransitionOnTimeout[1]      = "Ready";
    stateTimeoutValue[1]             = 0.725;
    stateSequence[1]                 = "ready";
    stateSound[1]                    = "ShotgunMountSound";

    stateName[2]                     = "Ready";
    stateTransitionOnNoAmmo[2]       = "Noammo";
    stateTransitionOnTriggerDown[2]  = "Fire";
    stateSequence[2]                 = "ready";
    stateSound[2]                    = "ShotgunIdleSound";
    stateTimeoutValue[2] = 1;
    stateTransitionOnTimeout[2] = "Ready2";
    stateEmitter[2] = "TurretGunBeamEmitter";
    stateEmitterTime[2] = 1;
    stateWaitForTimeout[2] = false;

    stateName[9]                     = "Ready2";
    stateTransitionOnNoAmmo[9]       = "Noammo";
    stateTransitionOnTriggerDown[9]  = "Fire";
    stateSequence[9]                 = "ready";
    stateSound[9]                    = "ShotgunIdleSound";
    stateTimeoutValue[9] = 1;
    stateTransitionOnTimeout[9] = "Ready";
    stateEmitter[9] = "TurretGunBeamEmitter";
    stateEmitterTime[9] = 1;
    stateWaitForTimeout[9] = false;

    stateName[3]                     = "Fire";
    stateTransitionOnTimeout[3]      = "Reload";
    stateTimeoutValue[3]             = 0.01;
    stateFire[3]                     = true;
    stateRecoil[3]                   = LightRecoil;
    stateAllowImageChange[3]         = false;
    stateSequence[3]                 = "Fire";
    stateScript[3]                   = "onFire";
    stateEjectShell[3]               = true;
    stateSound[3]                    = "ShotgunFireSound";
    stateEmitter[3] = ShotgunFlashEmitter;
    stateEmitterNode[3] = "muzzleNode";
    stateEmitterTime[3] = 0.1;

    stateName[4]                     = "Reload";
    stateTransitionOnAmmo[4]         = "doReload";
    stateTransitionOnNoAmmo[4]       = "NoAmmo";
    stateAllowImageChange[4]         = false;

    stateName[5]                     = "Noammo";
    stateTransitionOnAmmo[5]         = "doReload";
    stateSequence[5]                 = "Noammo";
    stateTransitionOnTriggerDown[5]  = "DryFire";

    stateName[6]                     = "Dryfire";
    stateSound[6]                    = "ShotgunDryfireSound";
    stateTimeoutValue[6]             = 1.0;
    stateSequence[6]                 = "Noammo";
    stateTransitionOnTimeout[6]      = "Noammo";

    stateName[7]                     = "CheckAmmo";
    stateTransitionOnAmmo[7]         = "Ready"; //"Activate";
    stateTransitionOnNoAmmo[7]       = "NoAmmo";
  
    stateName[8]                     = "doReload";
    stateTransitionOnTimeout[8]      = "Ready";
    stateTimeoutValue[8]             = 0.725;
    stateSequence[8]                 = "Reload";
    stateSound[8]                    = "ShotgunReloadSound";
    stateAllowImageChange[8]         = false;
};

// -----------------------------------------------------------------------------
// -- Functions
// -----------------------------------------------------------------------------

function TurretGunImage::onFire(%data, %player, %slot)
{
   if(%player.invincible)
      %player.schedule(150, setInvincible, false);
   %client = %player.client;

   %player.deploy(%data, turret);
}  

function Player::Deploy(%player, %weaponImageData, %deployableData)
{
   %client = %player.client;

   //personal limits for turrets?

   //team limits for turrets
   if(!TeamManager_SO.canDeploy(%client.team, %deployableData))
   {
      commandToClient(%client, 'centerPrint', "Limit", 1);
      return 0;
   }

   //raycast to target
   %start  = %player.getEyePoint();
   %vec = %player.getEyeVector();
   %end = vectorAdd(%start, vectorScale(%vec, 10));
   %mask = $TypeMasks::InteriorObjectType | $TypeMasks::TerrainObjectType;

   %search = containerRayCast(%start, %end, %mask);

   if(!%search)
   {
      commandToClient(%client, 'centerPrint', "No Target", 1);
      return 0;
   }

   %searchNormal = getWords(%search, 4,6);

   //validate slope
   %dot = vectorDot(%searchNormal, "0 0 1");
   %angle =  mAcos(%dot) * 180 / $pi;
   if(%angle > %deployableData.maxSlope)
   {
      commandToClient(%client, 'centerPrint', "Too Steep", 1);
      return 0;
   }

   //create turret object
   echo(%search);
   %pos = getWords(%search, 1, 3);
   %normal = getWords(%search, 4, 6);

   %t = new StaticShape()
   {
      dataBlock = %deployableData;
      client = %client;
      team = %client.team;
   };
   missionCleanup.add(%t);
   %t.setTransform(%pos);
   
   if(%deployableData.alignToNormal)
      %t.alignToNormal(%normal, %vec);

   //box clear check vs Static Geometry
   %center = %t.getWorldBoxCenter();   //recod
   
   if(%deployableData.staticClearRadius > 0)
   {
      %mask = $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType;
      %x = %y = %z = %deployableData.staticClearRadius;
      %t.setTransform("0 0 -1000"); //move turret out of the way so it doesnt block itself
      if(!containerBoxEmpty(%mask, %center, %x,%y,%z))
      {
         commandToClient(%client, 'centerPrint', "Blocked (Static Geometry)", 1);
         %t.delete();
         return 0;
      }
      %t.setTransform(%pos);   
   }

   //box clear check vs players
   if(%deployableData.playerClearRadius > 0)
   {
      %mask = $TypeMasks::PlayerObjectType;
      %x = %y = %z = %deployableData.playerClearRadius;
      if(!containerBoxEmpty(%mask, %center, %x,%y,%z))
      {
         commandToClient(%client, 'centerPrint', "Blocked (Player)", 1);
         %t.delete();
         return 0;
      }
   }
      
   if(isObject(%weaponImageData.ammo))
      %player.decInventory( %weaponImageData.ammo, 1 );

   %t.client = %client;
   %t.team = %client.team;

   TeamManager_SO.incDeployableCount(%t.team, %deployableData, 1);

   return %t;
}

// -----------------------------------------------------------------------------
//for some fucking reason this doesn't get called
function Turret::Scan(%data, %obj)
{
   //echo("turret scanning");

   if(isEventPending(%obj.scanEvent))
      cancel(%obj.scanEvent);

   //area search for players

   %pos = %obj.getWorldBoxCenter();
   %radius = %data.scanRadius;
   %mask = $TypeMasks::PlayerObjectType;
   //initContainerRadiusSearch(%pos, %radius, %mask);

   //shoot at nearest player
   %nearestObj = -1;
   %nearestDist = 99999;
   //while ((%searchObj = containerSearchNext()) != 0)
   %count = clientGroup.getCount();
   for(%i = 0; %i < %count; %i++)
   {
      %searchObj = clientGroup.getObject(%i).player;

      if(!isObject(%searchObj))
         continue;
      if(%searchObj.getDamagePercent() >= 1.0)
         continue;
      
      //dont shoot your teammates
      if(%searchObj.team == %obj.team)
         continue;

      %delta = vectorSub(%pos, %searchObj.getWorldBoxCenter());
      %dist = vectorLen(%delta);

      //dont shoot things out of range
      if(%dist > %data.scanRadius)
         continue;

      //dont shoot through at walls/terrain, should probably use the predicted target position for this
      %mask = $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType;
      %start = %obj.getWorldBoxCenter();
      %end = %searchObj.getWorldBoxCenter();
      %search = containerRayCast(%start,%end, %mask, %obj);
      if(%search)
         continue;

      //dont shoot through teammates
      %mask = $TypeMasks::PlayerObjectType;
      %start = %obj.getWorldBoxCenter();
      %end = %searchObj.getWorldBoxCenter();
      %search = getWord(containerRayCast(%start,%end, %mask, %obj), 0);
      if(%search.team == %obj.team)
         continue;

      //find the closest object
      if(%dist < %nearestDist)
      {
         %nearestObj = %searchObj;
         %nearestDist = %dist;
      }
   }

   if(%nearestObj == -1)
   {
      //couldn't find anything to shoot
      %obj.schedule(%data.scanTime, "Scan");
      return;
   }
   //echo(%nearestObj);
   //shoot at nearest object

   //anticipate where target is going to be
   %timeToTarget = %nearestDist / %data.projectileVelocity;
   //echo("time to target = ", %timetoTarget);
   %currTargetPos = %nearestObj.getWorldBoxCenter();
   %currTargetVel = %nearestObj.getVelocity();

   %targetVec = vectorScale(%currTargetVel, %timeToTarget);
   //echo("  target vel = ", %currTargetVel);
   //echo("  target vec = ", %targetVec);
   %shootPos = vectorAdd(%currTargetPos, %targetVec);

   %shootVec = vectorSub(%pos, %shootPos);
   %shootVec = vectorNormalize(%shootVec);
   %shootVec = vectorScale(%shootVec, -1 * %data.projectileVelocity);

   %p = new (%data.projectileType)()
   {
      dataBlock = %data.projectile;
      initialVelocity = %shootVec;
      initialPosition = %obj.getWorldBoxCenter();
      sourceObject = %obj;
      sourceSlot = 0;
      client = %obj.client;
   };
   MissionCleanup.add(%p);
  
   %obj.schedule(%data.scanTime, "Scan");
}  
//this doesnt get called either
function Turret::onAdd(%data, %obj)
{
   echo("turret onadd");
   %obj.scan();
}





