//zoneControl.cs

//zone control gameplay objects

// ------------------------------
// Flag Item
// ------------------------------
datablock ItemData( ZCFlagItem )
{
    category         = "Game";
    className        = "ZCFlag";
    dynamicType      = $TypeMasks::DamagableItemObjectType;

    shapeFile        = "~/data/shapes/items/flag_wht.dts";
    mass             = 50;
    elasticity       = 0.2;
    friction         = 0.6;

    throwForce       = 1000;

    flagImage = ZCFlagImage;
};
datablock ShapeBaseImageData( ZCFlagImage )
{
   className      = ZCFlagImageClass;
   isFlag         = true; //respond to drop flag command?
   shapeFile      = "~/data/shapes/items/flag_wht.dts";
   mountPoint     = 2;
};

function ZCFlag::GamePreProcess(%this, %obj)
{
   error("PRE PROCESSING ZCFLAG");
   TeamManager_SO.addFlag(%obj);
}

function ZCFlag::onAdd(%this, %obj)
{
   if(%obj.isStatic())
      %obj.status = "Safe";
}

function ZCFlag::OnPickup(%this,%obj,%player,%amount)
{
   if(%obj.ismounted())
      return;

   //echo("trying to pick up ZCflag");
   //echo("flag = ", %obj);
   //echo("player = ", %player);
 
   //are we carrying a flag?
   %haveFlag = false;
   for(%i = 0; %i < $MaxMountedImages; %i++)
   {
      %image = %player.getMountedImage(%i);
      if(!isObject(%image))
         continue;
      
      if(%image.isFlag == true)
         return;
   }

   //pickup flag
   %player.mountImage(%this.flagImage,$FlagImageSlot);
   if(%obj.isStatic())
   {
      //picking up flag at base
      %player.flagTeam = %obj.team;
      %player.flagSpawn = %obj;
      %player.flagDB = %this;
      %obj.setHidden(1);   
      %obj.holdingPlayer = %player;

      %player.client.incStat("Flag_StandGrabs");

      //echo(getTaggedString(%player.client.name) @ " STAND GRABBED " @ %player.flagspawn.getName());

      %player.flagSpawn.status = "Taken";
      %player.flagSpawn.transmitStatus();
      //messageAll('FlagStatus', '',  %player.flagSpawn, "Taken");
   }
   else
   {
      //don't count when it was dropped with ctrl F
      if(!%obj.wasPassed)
         %player.client.incStat("Flag_FieldGrabs");

      //picking up flag that has been dropped
      %player.flagTeam = %obj.team;
      %player.flagSpawn = %obj.spawn;
      %player.flagDB = %this;
      %obj.spawn.holdingPlayer = %player;

      //echo(getTaggedString(%player.client.name) @ " FIELD GRABBED " @ %player.flagspawn.getName());

      %obj.delete();

      %player.flagSpawn.status = "Taken";
      %player.flagSpawn.transmitStatus();
      //messageAll('FlagStatus', '',  %player.flagSpawn, "Taken");
   }
 
}

function ZCFlag::TransmitStatus(%this, %obj, %client)
{
   if(!%obj.isStatic())
   {
      error("ERROR: ZCFlag::TransmitStatus() - attempting to send status of non-static object \"" @ %obj @ "\"");
      return;
   }  
   
   switch$(%obj.status)
   {
      case "Safe":
         %status = "<color:FFFFFF><spush><font:wingdings:22>P<spop>";
      case "Taken":
         if(isObject(%obj.holdingPlayer))
         {
            //echo("holding player = ", %obj.holdingplayer.team);
            %color = TeamManager_SO.getTeamHexColor(%obj.holdingPlayer.team);
            %status = %color @ "<spush><font:wingdings:22>P<spop>";
            //%status = TeamManager_SO.teamName[%obj.holdingPlayer.team];
         }
         else
            %status = "<color:FF00FF><spush><font:wingdings:22>P<spop>";
      case "Dropped":
         %timeLeft = getEventTimeLeft(%obj.resetEvent);
         %status = "<color:000000><spush><font:wingdings:22>P<spop>";
         %status = %status TAB "COUNTDOWN" TAB %timeLeft;
      default:
         %status = "<color:FF00FF>?";
   }

   if(%client $= "")
      messageAll('FlagStatus', '', %obj, %status);
   else
      messageClient(%client, 'FlagStatus', '', %obj, %status);
}

function ZCFlag::Reset(%data, %obj)
{
   //echo("FlagItem resetting ", %data, " ", %obj);
   if(isObject(%obj.spawn))
   {
      %obj.spawn.status = "Safe";
      %obj.spawn.transmitStatus();
      //messageAll('FlagStatus', '', %obj.spawn, "Safe");
      %obj.spawn.setHidden(0);
      //%obj.schedule(10, delete);      
      %obj.delete();
   }
   else if(%obj.isStatic())
   {
      %obj.setHidden(0);
      %obj.status = "Safe";
      %obj.transmitStatus();
   }
   else
   {
      error("ERROR: FlagItem::Reset() - no spawn object for flag " @ %obj);
   }
}  

function ZCFlagImageClass::OnPlayerDeath(%data, %player, %slot)
{
   //echo("flag holder died ", %player);
   %player.unMountImage(%slot);

   %flag = FlagImageClass::DropFlag(%data,%player);  
   %flag.setCollisionTimeout(%player);
}
function ZCFlagImageClass::OnPlayerRemove(%data, %player, %slot)
{
   FlagImageClass::DropFlag(%data,%player);  
}


// ------------------------------
// Zone Trigger 
// ------------------------------

datablock TriggerData(ZoneControlTrigger)
{
   tickPeriodMS = 100;
};

function ZoneControlTrigger::GamePreProcess(%this, %obj)
{
   error("PRE PROCESSING ZONE CONTROL TRIGGER");

   //set initial status
   %obj.team = -1;

   //add self to flag list
   TeamManager_SO.addFlag(%obj);

   //add status light
   %obj.light = new FxLight()
   {
      dataBlock = whiteLight;
   };
   MissionCleanup.add(%obj.light);
   %obj.light.setTransform(%obj.getWorldBoxCenter());
}

function Trigger::TransmitStatus(%trigger)
{
   %trigger.getDataBlock().transmitStatus(%trigger);
}
function ZoneControlTrigger::TransmitStatus(%this, %obj, %client)
{
   if(%obj.team == -1)
      %status = "<color:FFFFFF><spush><font:webdings:20>H<spop>";
   else
   {
      %color = TeamManager_SO.getTeamHexColor(%obj.team);
      %status = %color @ "<spush><font:webdings:20>H<spop>";
      //%status = TeamManager_SO.teamName[%obj.team];
   }

   

   if(%client $= "")
      messageAll('FlagStatus', '', %obj, %status);
   else
      messageClient(%client, 'FlagStatus', '', %obj, %status);   
}

function ZoneControlTrigger::onEnterTrigger(%this,%trigger,%obj)
{
   Parent::onEnterTrigger(%this,%trigger,%obj);
   
   //must be player with a client and with the ZCFlag
   if(!(%obj.getType() & $TypeMasks::PlayerObjectType))
      return;
   %client = %obj.client;
   if(!isObject(%client))
      return;

   //do we have the ZCFlag?
   %image = %obj.getMountedImage($FlagImageSlot);
   if(!isObject(%image))  
      return;
   if(%image.className !$= "ZCFlagImageClass")
      return;
   
   //no need to do anything if it's already capped
   if(%trigger.team == %client.team)
      return;
   
   //shuffle points accordingly
   if(%trigger.team != -1)
      TeamManager_SO.incScore(%trigger.team, -1);
   %trigger.setTeam(%client.team);
   TeamManager_SO.incScore(%trigger.team, 1);

   //if all zones are on the same team now, reset everything
   TeamManager_SO.checkAllZones(%obj);
}

function ZoneControlTrigger::SetTeam(%this, %trigger, %team)
{
   %trigger.team = %team;
   
   //echo("setting zone " @ %trigger @ " to team " @ %team);

   switch(%team)
   {
      case -1:
         %lightData = WhiteLight;
      case 0:
         %lightData = RedLight;
      case 1:
         %lightData = BlueLight;
      default:
         error("ERROR: ZoneControlTrigger::SetTeam() - Unknown team \"" @ %team @ "\"");
   }
   %trigger.light.setDataBlock(%lightData);

   %trigger.transmitStatus();
}

function ZoneControlTrigger::onLeaveTrigger(%this,%trigger,%obj)
{
   Parent::onLeaveTrigger(%this,%trigger,%obj);
}

function ZoneControlTrigger::onTickTrigger(%this,%trigger, %obj)
{  
   Parent::onTickTrigger(%this,%trigger);
}



// ------------------------------------
// Extensions to Team Manager
// ------------------------------------
//check if all of the zone control zones are on the same team
function TeamManager_SO::CheckAllZones(%this, %player)
{
   %count = getWordCount(%this.flagList);

   %currTeam = 999;

   for(%i = 0; %i < %count; %i++)
   {
      %flag = getWord(%this.flagList, %i);

      if(!(%flag.getType() & $TypeMasks::TriggerObjectType))
         continue;
      if(%flag.getDataBlock().getID() != ZoneControlTrigger.getID())
         continue;

      if(%currTeam == 999)
         %currTeam = %flag.team;
      else
         if(%currTeam != %flag.team) 
            return;  //there is at least one zone that is not the same as the others
   }

   if(%currTeam == 999)
   {
      Error("ERROR: TeamManager_SO::CheckAllZones() - no zones found, this should not happen");
   }
   
   //respawn the flag
   %player.unMountImage($FlagImageSlot);
   %player.flagSpawn.schedule(5000, reset);

   //reset all zones in 
   %this.schedule(5000, resetZones);
}

//reset all the zoneControl Triggers to neutral team and respawn the ZCFlags
function TeamManager_SO::ResetZones(%this)
{
   echo("TeamManager - Resetting ZC zones");   
   %count = getWordCount(%this.flagList);

   %currTeam = 999;

   for(%i = 0; %i < %count; %i++)
   {
      %flag = getWord(%this.flagList, %i);

      if(!(%flag.getType() & $TypeMasks::TriggerObjectType))
         continue;
      if(%flag.getDataBlock().getID() != ZoneControlTrigger.getID())
         continue;
      
      //echo("  resetting zone ", %flag);
      %flag.setTeam(-1);
   }  
}