//-----------------------------------------------------------------------------
// Legions
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------


$defaultEditor = "creator";
$defaultGame = "base";
$displayHelp = false;

$userGame = $defaultGame;
$isDedicated = false;

for ($i = 1; $i < $Game::argc ; $i++)
{
   $arg = $Game::argv[$i];
   $nextArg = $Game::argv[$i + 1];
   $hasNextArg = $Game::argc - $i > 1;
   $logModeSpecified = false;

   // check for dedicated run
   if( stricmp($arg, "-dedicated") == 0  )
      $isDedicated = true;

   switch$ ($arg)
   {
      //--------------------
      case "-log":
         $argUsed[$i]++;
         if ($hasNextArg)
         {
            // turn on console logging
            if ($nextArg != 0)
            {
               // dump existing console to logfile first
               $nextArg += 4;
            }
            
            setLogMode($nextArg);
            $logModeSpecified = true;
            $argUsed[$i + 1]++;
            $i++;
         }
         else
            error("Error: Missing Command Line argument. Usage: -log <Mode: 0,1,2>");

      //--------------------
      case "-game":
         $argUsed[$i]++;
         if ($hasNextArg)
         {
            // set the selected game
            $userGame = $nextArg;
            
            $argUsed[$i + 1]++;
            $i++;
            error($userMods);
         }
         else
            error("Error: Missing Command Line argument. Usage: -game <game_name>");

      //--------------------
      case "-console":
         enableWinConsole(true);
         $argUsed[$i]++;

      //--------------------
      case "-jSave":
         $argUsed[$i]++;
         if ($hasNextArg)
         {
            echo("Saving event log to journal: " @ $nextArg);
            
            saveJournal($nextArg);
            $argUsed[$i + 1]++;
            $i++;
         }
         else
            error("Error: Missing Command Line argument. Usage: -jSave <journal_name>");

      //--------------------
      case "-jPlay":
         $argUsed[$i]++;
         if ($hasNextArg)
         {
            playJournal($nextArg, false);
            $argUsed[$i + 1]++;
            $i++;
         }
         else
            error("Error: Missing Command Line argument. Usage: -jPlay <journal_name>");

      //--------------------
      case "-jDebug":
         $argUsed[$i]++;
         if ($hasNextArg)
         {
            playJournal($nextArg,true);
            $argUsed[$i + 1]++;
            $i++;
         }
         else
            error("Error: Missing Command Line argument. Usage: -jDebug <journal_name>");

      //-------------------
      case "-help":
         $displayHelp = true;
         $argUsed[$i]++;

      //-------------------
      default:
         // do nothing
   }
}


//-----------------------------------------------------------------------------


function onStart()
{
   // Default startup function
}

function onExit()
{
   // OnExit is called directly from C++ code, whereas onStart is
   // invoked at the end of this file.
}

function parseArgs()
{
   // Here for mod override, the arguments have already
   // been parsed.
}

package Help
{
   function onExit()
   {
      // Override onExit when displaying help
   }
};

function displayHelp()
{
   activatePackage(Help);

      // Notes on logmode: console logging is written to console.log.
      // -log 0 disables console logging.
      // -log 1 appends to existing logfile; it also closes the file
      // (flushing the write buffer) after every write.
      // -log 2 overwrites any existing logfile; it also only closes
      // the logfile when the application shuts down.  (default)

   error(
      "Torque Demo command line options:\n" @
      "  -log <logmode>         Logging behavior; see main.cs comments for details\n" @
      "  -game <game_name>      Reset list of mods to only contain <game_name>\n" @
      "  -console               Open a separate console\n" @
      "  -jSave  <file_name>    Record a journal\n" @
      "  -jPlay  <file_name>    Play back a journal\n" @
      "  -jDebug <file_name>    Play back a journal and issue an int3 at the end\n" @
      "  -help                  Display this help message\n"
   );
}


//-----------------------------------------------------------------------------


// Default to a new logfile each session.
if (!$logModeSpecified)
{
   if ($platform !$= "xbox" && $platform !$= "xenon")
      setLogMode(6);
}

function loadGame()
{
   %mods = $userGame;
   if (!$isDedicated)
      %mods = $defaultEditor @ ";" @ $userGame;
   
   echo("--------- Loading GAME ---------");
   setModPaths(%mods);
   
   
   if (exec($userGame @ "/main.cs") != true)
      error("Error: Unable to find specified game: " @ $userGame);
   
   if (!$isDedicated)
   {
      if (exec($defaultEditor @ "/main.cs") != true)
         error("Error: Unable to find the editor: " @ $defaultEditor);
   }
}

loadGame();
echo("");

// Parse the command line arguments
echo("--------- Parsing Arguments ---------");
parseArgs();

// Either display the help message or startup the app.
if ($displayHelp)
{
   enableWinConsole(true);
   displayHelp();
   quit();
}
else
{
   onStart();
   echo("Engine initialized...");
}

// Display an error message for unused arguments
for ($i = 1; $i < $Game::argc; $i++)
{
   if (!$argUsed[$i])
      error("Error: Unknown command line argument: " @ $Game::argv[$i]);
}
