//*****************************************************************************
// TGEA -- HLSL procedural shader                                              
//*****************************************************************************
//-----------------------------------------------------------------------------
// Structures                                                                  
//-----------------------------------------------------------------------------
struct ConnectData
{
   float2 texCoord        : TEXCOORD0;
   float4 dlightCoord     : TEXCOORD1;
   float4 dlightCoordSec  : TEXCOORD2;
   float3 lightVec        : TEXCOORD3;
   float3 lightVecSec     : TEXCOORD4;
   float2 fogCoord        : TEXCOORD5;
};


struct Fragout
{
   float4 col : COLOR0;
};


//-----------------------------------------------------------------------------
// Main                                                                        
//-----------------------------------------------------------------------------
Fragout main( ConnectData IN,
              uniform sampler2D diffuseMap      : register(S0),
              uniform sampler3D dlightMap       : register(S1),
              uniform float4    lightColor      : register(C2),
              uniform sampler3D dlightMapSec    : register(S2),
              uniform float4    lightColorSec   : register(C5),
              uniform sampler2D bumpMap         : register(S3),
              uniform float4    ambient         : register(C3),
              uniform sampler2D fogMap          : register(S4)
)
{
   Fragout OUT;

   float4 diffuseColor = tex2D(diffuseMap, IN.texCoord);
   OUT.col = diffuseColor;
   OUT.col.a = 1.0;
   float4 attn = tex3D(dlightMap, IN.dlightCoord) * lightColor;
   float4 attnsec = tex3D(dlightMapSec, IN.dlightCoordSec) * lightColorSec;
   OUT.col *= attn;
   float4 bumpNormal = tex2D(bumpMap, IN.texCoord);

   float4 bumpDot = saturate( dot(bumpNormal.xyz * 2.0 - 1.0, normalize(IN.lightVec.xyz)) * IN.dlightCoord.w );
   attnsec *= saturate( dot(bumpNormal.xyz * 2.0 - 1.0, normalize(IN.lightVecSec.xyz)) * IN.dlightCoordSec.w );
   OUT.col *= bumpDot;
   OUT.col += (attnsec * diffuseColor);
   float4 fogColor = tex2D(fogMap, IN.fogCoord);
   OUT.col.a = 1.0 - fogColor.a;

   return OUT;
}
